@@ -1,7 +1,23 @@
 /*
- * $Id: calendar.c 7939 2009-10-23 12:51:53Z dothebart $
+ * $Id: calendar.c 8595 2010-05-25 04:05:05Z ajc $
  *
  * Functions which handle calendar objects and their processing/display.
+ *
+ * Copyright (c) 1996-2010 by the citadel.org team
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "webcit.h"
@@ -885,6 +901,7 @@ InitModule_CALENDAR
 		VIEW_CALENDAR,
 		calendar_GetParamsGetServerCall,
 		NULL,
+		NULL,
 		calendar_LoadMsgFromServer,
 		calendar_RenderView_or_Tail,
 		calendar_Cleanup);
@@ -893,6 +910,7 @@ InitModule_CALENDAR
 		VIEW_CALBRIEF,
 		calendar_GetParamsGetServerCall,
 		NULL,
+		NULL,
 		calendar_LoadMsgFromServer,
 		calendar_RenderView_or_Tail,
 		calendar_Cleanup);
