@@ -1,11 +1,26 @@
 /*
- * $Id: context_loop.c 8109 2009-12-10 22:16:58Z dothebart $
+ * $Id: context_loop.c 8676 2010-07-12 17:16:45Z ajc $
  *
  * This is the other half of the webserver.  It handles the task of hooking
  * up HTTP requests with the sessions they belong to, using HTTP cookies to
  * keep track of things.  If the HTTP request doesn't belong to any currently
  * active session, a new session is started.
  *
+ * Copyright (c) 1996-2010 by the citadel.org team
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "webcit.h"
@@ -21,7 +36,10 @@ pthread_key_t MyConKey;         /* TSD key for MySession() */
 HashList *HttpReqTypes = NULL;
 HashList *HttpHeaderHandler = NULL;
 extern HashList *HandlerHash;
-int num_threads = 1;		/* Number of worker threads.  Start at 1 because the parent counts. */
+
+/* the following two values start at 1 because the initial parent thread counts as one. */
+int num_threads_existing = 1;		/* Number of worker threads which exist. */
+int num_threads_executing = 1;		/* Number of worker threads currently executing. */
 
 void DestroyHttpHeaderHandler(void *V)
 {
@@ -44,16 +62,13 @@ void do_housekeeping(void)
 {
 	wcsession *sptr, *ss;
 	wcsession *sessions_to_kill = NULL;
-	int num_sessions = 0;
 
 	/*
 	 * Lock the session list, moving any candidates for euthanasia into
 	 * a separate list.
 	 */
 	pthread_mutex_lock(&SessionListMutex);
-	num_sessions = 0;
 	for (sptr = SessionList; sptr != NULL; sptr = sptr->next) {
-		++num_sessions;
 
 		/* Kill idle sessions */
 		if ((time(NULL) - (sptr->lastreq)) > (time_t) WEBCIT_TIMEOUT) {
@@ -90,18 +105,28 @@ void do_housekeeping(void)
 
 		session_destroy_modules(&sessions_to_kill);
 		sessions_to_kill = sptr;
-		--num_sessions;
 	}
+}
 
-	/*
-	 * If there are more sessions than threads, then we should spawn
-	 * more threads ... up to a predefined maximum.
-	 */
-	while ( (num_sessions > num_threads) && (num_threads <= MAX_WORKER_THREADS) ) {
+/*
+ * Check the size of our thread pool.  If all threads are executing, spawn another.
+ */
+void check_thread_pool_size(void)
+{
+	if (time_to_die) return;		/* don't expand the thread pool during shutdown */
+
+	begin_critical_section(S_SPAWNER);	/* only one of these should run at a time */
+	while (
+		(num_threads_executing >= num_threads_existing)
+		&& (num_threads_existing < MAX_WORKER_THREADS)
+	) {
+		lprintf(3, "%d of %d threads are executing.  Adding another worker thread.\n",
+			num_threads_executing,
+			num_threads_existing
+		);
 		spawn_another_worker_thread();
-		lprintf(3, "There are %d sessions and %d threads active.\n",
-			num_sessions, num_threads);
 	}
+	end_critical_section(S_SPAWNER);
 }
 
 
@@ -187,7 +212,6 @@ wcsession *CreateSession(int Lockable, int Static, wcsession **wclist, ParsedHtt
 	TheSession->Hdr = Hdr;
 	TheSession->SessionKey = Hdr->HR.SessionKey;
 	TheSession->serv_sock = (-1);
-	TheSession->chat_sock = (-1);
 	TheSession->is_mobile = -1;
 
 	pthread_setspecific(MyConKey, (void *)TheSession);
@@ -445,8 +469,13 @@ void OverrideRequest(ParsedHttpHdrs *Hdr, const char *Line, long len)
 {
 	StrBuf *Buf = NewStrBuf();
 
-	FlushStrBuf(Hdr->HR.ReqLine);
-	StrBufPlain(Hdr->HR.ReqLine, Line, len);
+	if (Hdr->HR.ReqLine != NULL) {
+		FlushStrBuf(Hdr->HR.ReqLine);
+		StrBufPlain(Hdr->HR.ReqLine, Line, len);
+	}
+	else {
+		Hdr->HR.ReqLine = NewStrBufPlain(Line, len);
+	}
 	ReadHttpSubject(Hdr, Hdr->HR.ReqLine, Buf);
 
 	FreeStrBuf(&Buf);
@@ -615,11 +644,6 @@ void tmplput_current_user(StrBuf *Target, WCTemplputParams *TP)
 	StrBufAppendTemplate(Target, TP, WC->wc_fullname, 0);
 }
 
-void tmplput_current_room(StrBuf *Target, WCTemplputParams *TP)
-{
-	StrBufAppendTemplate(Target, TP, WC->wc_roomname, 0); 
-}
-
 void Header_HandleContentLength(StrBuf *Line, ParsedHttpHdrs *hdr)
 {
 	hdr->HR.ContentLength = StrToi(Line);
@@ -775,7 +799,6 @@ InitModule_CONTEXT
 	RegisterHeaderHandler(HKEY("IF-MODIFIED-SINCE"), Header_HandleIfModSince);
 
 	RegisterNamespace("CURRENT_USER", 0, 1, tmplput_current_user, NULL, CTX_NONE);
-	RegisterNamespace("CURRENT_ROOM", 0, 1, tmplput_current_room, NULL, CTX_NONE);
 	RegisterNamespace("NONCE", 0, 0, tmplput_nonce, NULL, 0);
 
 	WebcitAddUrlHandler(HKEY("404"), "", 0, do_404, ANONYMOUS|COOKIEUNNEEDED);
