@@ -1,4 +1,4 @@
-/* $Id: groupdav.h 7804 2009-09-10 21:27:55Z dothebart $ */
+/* $Id: groupdav.h 8515 2010-04-19 10:20:54Z dothebart $ */
 
 
 /*
@@ -27,3 +27,11 @@ void euid_escapize(char *, const char *);
 void euid_unescapize(char *, const char *);
 void groupdav_identify_host(void);
 void groupdav_identify_hosthdr(void);
+
+void RegisterDAVNamespace(const char * UrlString, 
+			  long UrlSLen, 
+			  const char *DisplayName, 
+			  long dslen, 
+			  WebcitHandlerFunc F, 
+			  WebcitRESTDispatchID RID,
+			  long Flags);
