@@ -1,5 +1,5 @@
 /*
- * $Id: messages.c 8102 2009-12-03 23:00:40Z ajc $
+ * $Id: messages.c 8624 2010-06-17 19:36:27Z dothebart $
  *
  * Functions which deal with the fetching and displaying of messages.
  *
@@ -69,13 +69,12 @@ int load_message(message_summary *Msg,
 		{
 			Done = 1;
 			if (state < 2) {
-				lprintf(1, _("unexpected end of message"));
 				if (Msg->MsgBody->Data == NULL)
 					Msg->MsgBody->Data = NewStrBuf();
 				Msg->MsgBody->ContentType = NewStrBufPlain(HKEY("text/html"));
 				StrBufAppendPrintf(Msg->MsgBody->Data, "<div><i>");
-				StrBufAppendPrintf(Msg->MsgBody->Data, _("unexpected end of message"));
-				StrBufAppendPrintf(Msg->MsgBody->Data, " (1)</i><br /><br />\n");
+				StrBufAppendPrintf(Msg->MsgBody->Data, _("Empty message"));
+				StrBufAppendPrintf(Msg->MsgBody->Data, "</i><br /><br />\n");
 				StrBufAppendPrintf(Msg->MsgBody->Data, "</div>\n");
 			}
 			break;
@@ -382,7 +381,7 @@ void handle_one_message(void)
 		break;
 	case eDELETE:
 		CmdBuf = NewStrBuf ();
-		if (WCC->wc_is_trash) {	/* Delete from Trash is a real delete */
+		if ((WCC->CurRoom.RAFlags & UA_ISTRASH) != 0) {	/* Delete from Trash is a real delete */
 			serv_printf("DELE %ld", msgnum);	
 		}
 		else {			/* Otherwise move it to Trash */
@@ -442,7 +441,7 @@ void embed_message(void) {
 		break;
 	case eDELETE:
 		CmdBuf = NewStrBuf ();
-		if (WCC->wc_is_trash) {	/* Delete from Trash is a real delete */
+		if ((WCC->CurRoom.RAFlags & UA_ISTRASH) != 0) {	/* Delete from Trash is a real delete */
 			serv_printf("DELE %ld", msgnum);	
 		}
 		else {			/* Otherwise move it to Trash */
@@ -572,9 +571,10 @@ message_summary *ReadOneMessageSummary(StrBuf *RawMessage, const char *DefaultSu
  *
  * servcmd:		the citadel command to send to the citserver
  */
-int load_msg_ptrs(const char *servcmd, SharedMessageStatus *Stat)
+int load_msg_ptrs(const char *servcmd, 
+		  SharedMessageStatus *Stat, 
+		  load_msg_ptrs_detailheaders LH)
 {
-	StrBuf* FoundCharset = NULL;
         wcsession *WCC = WC;
 	message_summary *Msg;
 	StrBuf *Buf, *Buf2;
@@ -633,47 +633,11 @@ int load_msg_ptrs(const char *servcmd, SharedMessageStatus *Stat)
 				if (StrLength(Buf) < 32) 
 					skipit = 1;
 			}
-			if (!skipit) {
-				Msg->from = NewStrBufPlain(NULL, StrLength(Buf));
-				StrBufExtract_NextToken(Buf2, Buf, &Ptr, '|');
-				if (StrLength(Buf2) != 0) {
-					/* Handle senders with RFC2047 encoding */
-					StrBuf_RFC822_to_Utf8(Msg->from, Buf2, WCC->DefaultCharset, FoundCharset);
-				}
-			
-				/* node name */
-				StrBufExtract_NextToken(Buf2, Buf, &Ptr, '|');
-				if ((StrLength(Buf2) !=0 ) &&
-				    ( ((WCC->room_flags & QR_NETWORK)
-				       || ((strcasecmp(ChrPtr(Buf2), ChrPtr(WCC->serv_info->serv_nodename))
-					    && (strcasecmp(ChrPtr(Buf2), ChrPtr(WCC->serv_info->serv_fqdn))))))))
-				{
-					StrBufAppendBufPlain(Msg->from, HKEY(" @ "), 0);
-					StrBufAppendBuf(Msg->from, Buf2, 0);
-				}
-
-				/* Internet address (not used)
-				 *	StrBufExtract_token(Msg->inetaddr, Buf, 4, '|');
-				 */
-				StrBufSkip_NTokenS(Buf, &Ptr, '|', 1);
-				Msg->subj = NewStrBufPlain(NULL, StrLength(Buf));
-				StrBufExtract_NextToken(Buf2,  Buf, &Ptr, '|');
-				if (StrLength(Buf2) == 0)
-					StrBufAppendBufPlain(Msg->subj, _("(no subject)"), -1,0);
-				else {
-					StrBuf_RFC822_to_Utf8(Msg->subj, Buf2, WCC->DefaultCharset, FoundCharset);
-					if ((StrLength(Msg->subj) > 75) && 
-					    (StrBuf_Utf8StrLen(Msg->subj) > 75)) {
-						StrBuf_Utf8StrCut(Msg->subj, 72);
-						StrBufAppendBufPlain(Msg->subj, HKEY("..."), 0);
-					}
-				}
-
-				if ((StrLength(Msg->from) > 25) && 
-				    (StrBuf_Utf8StrLen(Msg->from) > 25)) {
-					StrBuf_Utf8StrCut(Msg->from, 23);
-					StrBufAppendBufPlain(Msg->from, HKEY("..."), 0);
-				}
+			if ((!skipit) && (LH != NULL)) {
+				if (!LH(Buf, &Ptr, Msg, Buf2)){
+					free(Msg);
+					continue;
+				}					
 			}
 			n = Msg->msgnum;
 			Put(WCC->summ, (const char *)&n, sizeof(n), Msg, DestroyMessageSummary);
@@ -686,17 +650,6 @@ int load_msg_ptrs(const char *servcmd, SharedMessageStatus *Stat)
 }
 
 
-inline message_summary* GetMessagePtrAt(int n, HashList *Summ)
-{
-	const char *Key;
-	long HKLen;
-	void *vMsg;
-
-	if (Summ == NULL)
-		return NULL;
-	GetHashAt(Summ, n, &HKLen, &Key, &vMsg);
-	return (message_summary*) vMsg;
-}
 
 
 
@@ -745,15 +698,16 @@ typedef struct _RoomRenderer{
 	LoadMsgFromServer_func LoadMsgFromServer;
 	RenderView_or_Tail_func RenderView_or_Tail;
 	View_Cleanup_func ViewCleanup;
+	load_msg_ptrs_detailheaders LHParse;
 } RoomRenderer;
 
 
 /*
  * command loop for reading messages
  *
- * Set oper to "readnew" or "readold" or "readfwd" or "headers" or "readgt" or "do_search"
+ * Set oper to "readnew" or "readold" or "readfwd" or "headers" or "readgt" or "readlt" or "do_search"
  */
-void readloop(long oper)
+void readloop(long oper, eCustomRoomRenderer ForceRenderer)
 {
 	RoomRenderer *ViewMsg;
 	void *vViewMsg;
@@ -770,14 +724,14 @@ void readloop(long oper)
 	void *ViewSpecific;
 
 	if (havebstr("is_summary") && (1 == (ibstr("is_summary")))) {
-		WCC->wc_view = VIEW_MAILBOX;
+		WCC->CurRoom.view = VIEW_MAILBOX;
 	}
 
 	if (havebstr("is_ajax") && (1 == (ibstr("is_ajax")))) {
 		WCC->is_ajax = 1;
 	}
 
-	if ((oper == do_search) && (WCC->wc_view == VIEW_WIKI)) {
+	if ((oper == do_search) && (WCC->CurRoom.view == VIEW_WIKI)) {
 		display_wiki_pagelist();
 		return;
 	}
@@ -786,10 +740,13 @@ void readloop(long oper)
 	Stat.maxload = 10000;
 	Stat.lowest_found = (-1);
 	Stat.highest_found = (-1);
-	GetHash(ReadLoopHandler, IKEY(WCC->wc_view), &vViewMsg);
+	if (ForceRenderer == eUseDefault)
+		GetHash(ReadLoopHandler, IKEY(WCC->CurRoom.view), &vViewMsg);
+	else 
+		GetHash(ReadLoopHandler, IKEY(ForceRenderer), &vViewMsg);
 	if (vViewMsg == NULL) {
-		WCC->wc_view = VIEW_BBS;
-		GetHash(ReadLoopHandler, IKEY(WCC->wc_view), &vViewMsg);
+		WCC->CurRoom.view = VIEW_BBS;
+		GetHash(ReadLoopHandler, IKEY(WCC->CurRoom.view), &vViewMsg);
 	}
 	if (vViewMsg == NULL) {
 		return;			// TODO: print message
@@ -798,7 +755,7 @@ void readloop(long oper)
 	ViewMsg = (RoomRenderer*) vViewMsg;
 	if (!WCC->is_ajax) {
 		output_headers(1, 1, 1, 0, 0, 0);
-	} else if (WCC->wc_view == VIEW_MAILBOX) {
+	} else if (WCC->CurRoom.view == VIEW_MAILBOX) {
 		jsonMessageListHdr();
 	}
 
@@ -825,7 +782,7 @@ void readloop(long oper)
 		break;
 	}
 	if (!IsEmptyStr(cmd))
-		Stat.nummsgs = load_msg_ptrs(cmd, &Stat);
+		Stat.nummsgs = load_msg_ptrs(cmd, &Stat, ViewMsg->LHParse);
 
 	if (Stat.sortit) {
 		CompareFunc SortIt;
@@ -837,8 +794,9 @@ void readloop(long oper)
 		if (SortIt != NULL)
 			SortByPayload(WCC->summ, SortIt);
 	}
-	if (Stat.startmsg < 0) 
-		Stat.startmsg = (Stat.reverse) ? Stat.nummsgs - 1 : 0;
+	if (Stat.startmsg < 0) {
+		Stat.startmsg =  0;
+	}
 
 	if (Stat.load_seen) load_seen_flags();
 	
@@ -855,7 +813,7 @@ void readloop(long oper)
 	/* Put some helpful data in vars for mailsummary_json */
 	svputlong("READLOOP:TOTALMSGS", Stat.nummsgs);
 	svputlong("READLOOP:STARTMSG", Stat.startmsg);
-	svputlong("WCVIEW", WCC->wc_view);
+	svputlong("WCVIEW", WCC->CurRoom.view);
 
 	/*
 	 * iterate over each message. if we need to load an attachment, do it here. 
@@ -931,7 +889,7 @@ void post_mime_to_server(void) {
 	}
 
 	/* Only do multipart/alternative for mailboxes.  BBS and Wiki rooms don't need it. */
-	if (WC->wc_view == VIEW_MAILBOX) {
+	if (WC->CurRoom.view == VIEW_MAILBOX) {
 		include_text_alt = 1;
 	}
 
@@ -1018,6 +976,9 @@ void post_mime_to_server(void) {
  */
 void post_message(void)
 {
+	StrBuf *UserName;
+	StrBuf *EmailAddress;
+	StrBuf *EncBuf;
 	char buf[1024];
 	StrBuf *encoded_subject = NULL;
 	static long dont_post = (-1L);
@@ -1042,7 +1003,9 @@ void post_message(void)
 	if (WCC->upload_length > 0) {
 		const char *pch;
 		int n;
-		char N[64];
+		const char *newn;
+		long newnlen;
+		void *v;
 
 		/* There's an attachment.  Save it to this struct... */
 		lprintf(9, "Client is uploading %d bytes\n", WCC->upload_length);
@@ -1050,15 +1013,20 @@ void post_message(void)
 		memset(att, 0, sizeof(wc_mime_attachment ));
 		att->length = WCC->upload_length;
 		att->ContentType = NewStrBufPlain(WCC->upload_content_type, -1);
-		att->FileName = NewStrBufPlain(WCC->upload_filename, -1);
+		att->FileName = NewStrBufDup(WCC->upload_filename);
 		
 		if (WCC->attachments == NULL) {
-			WCC->attachments = NewHash(1, NULL);
+			WCC->attachments = NewHash(1, Flathash);
 		}
 
 		/* And add it to the list. */
-		n = snprintf(N, sizeof N, "%d", GetCount(WCC->attachments) + 1);
-		Put(WCC->attachments, N, n, att, DestroyMime);
+		n = 0;
+		if ((GetCount(WCC->attachments) > 0) && 
+		    GetHashAt(WCC->attachments, 
+			      GetCount(WCC->attachments) -1, 
+			      &newnlen, &newn, &v))
+		    n = *((int*) newn) + 1;
+		Put(WCC->attachments, IKEY(n), att, DestroyMime);
 
 		/*
 		 * Mozilla sends a simple filename, which is what we want,
@@ -1094,17 +1062,45 @@ void post_message(void)
 	} else if (lbstr("postseq") == dont_post) {
 		sprintf(WCC->ImportantMessage, 
 			_("Automatically cancelled because you have already "
-			"saved this message."));
+			  "saved this message."));
+	} else if (havebstr("remove_attach_button")) {
+		/* now thats st00pit. need to find it by name. */
+		void *vAtt;
+		StrBuf *WhichAttachment;
+		HashPos *at;
+		long len;
+		const char *key;
+
+		WhichAttachment = NewStrBufDup(sbstr("which_attachment"));
+		StrBufUnescape(WhichAttachment, 0);
+		at = GetNewHashPos(WCC->attachments, 0);
+		do {
+			GetHashPos(WCC->attachments, at, &len, &key, &vAtt);
+		
+			att = (wc_mime_attachment*) vAtt;
+			if ((att != NULL) && 
+			    (strcmp(ChrPtr(WhichAttachment), 
+				    ChrPtr(att->FileName)   ) == 0))
+			{
+				DeleteEntryFromHash(WCC->attachments, at);
+				break;
+			}
+		}
+		while (NextHashPos(WCC->attachments, at));
+		FreeStrBuf(&WhichAttachment);
+		display_enter();
+		return;
 	} else {
 		const char CMD[] = "ENT0 1|%s|%d|4|%s|%s||%s|%s|%s|%s|%s";
-		const StrBuf *Recp = NULL; 
-		const StrBuf *Cc = NULL;
-		const StrBuf *Bcc = NULL;
+		StrBuf *Recp = NULL; 
+		StrBuf *Cc = NULL;
+		StrBuf *Bcc = NULL;
 		const StrBuf *Wikipage = NULL;
 		const StrBuf *my_email_addr = NULL;
 		StrBuf *CmdBuf = NULL;
 		StrBuf *references = NULL;
 		int save_to_drafts;
+		long HeaderLen;
 
 		save_to_drafts = havebstr("save_button");
 		Buf = NewStrBuf();
@@ -1144,22 +1140,29 @@ void post_message(void)
 			
 			StrBufRFC2047encode(&encoded_subject, Subj);
 		}
-		Recp = sbstr("recp");
-		Cc = sbstr("cc");
-		Bcc = sbstr("bcc");
+		UserName = NewStrBuf();
+		EmailAddress = NewStrBuf();
+		EncBuf = NewStrBuf();
+
+		Recp = StrBufSanitizeEmailRecipientVector(sbstr("recp"), UserName, EmailAddress, EncBuf);
+		Cc = StrBufSanitizeEmailRecipientVector(sbstr("cc"), UserName, EmailAddress, EncBuf);
+		Bcc = StrBufSanitizeEmailRecipientVector(sbstr("bcc"), UserName, EmailAddress, EncBuf);
+
+		FreeStrBuf(&UserName);
+		FreeStrBuf(&EmailAddress);
+		FreeStrBuf(&EncBuf);
+
 		Wikipage = sbstr("page");
 		my_email_addr = sbstr("my_email_addr");
 		
-		CmdBuf = NewStrBufPlain(NULL, 
-					sizeof (CMD) + 
-					StrLength(Recp) + 
-					StrLength(encoded_subject) +
-					StrLength(Cc) +
-					StrLength(Bcc) + 
-					StrLength(Wikipage) +
-					StrLength(my_email_addr) + 
-					StrLength(references));
-
+		HeaderLen = StrLength(Recp) + 
+			StrLength(encoded_subject) +
+			StrLength(Cc) +
+			StrLength(Bcc) + 
+			StrLength(Wikipage) +
+			StrLength(my_email_addr) + 
+			StrLength(references);
+		CmdBuf = NewStrBufPlain(NULL, sizeof (CMD) + HeaderLen);
 		StrBufPrintf(CmdBuf, 
 			     CMD,
 			     save_to_drafts?"":ChrPtr(Recp),
@@ -1174,52 +1177,65 @@ void post_message(void)
 		FreeStrBuf(&references);
 		FreeStrBuf(&encoded_subject);
 
-		lprintf(9, "%s\n", ChrPtr(CmdBuf));
-		serv_puts(ChrPtr(CmdBuf));
-		FreeStrBuf(&CmdBuf);
+		if ((HeaderLen + StrLength(sbstr("msgtext")) < 10) && 
+		    (GetCount(WCC->attachments) == 0)){
+			StrBufAppendBufPlain(WCC->ImportantMsg, _("Refusing to post empty message.\n"), -1, 0);
+			FreeStrBuf(&CmdBuf);
+				
+		}
+		else 
+		{
+			lprintf(9, "%s\n", ChrPtr(CmdBuf));
+			serv_puts(ChrPtr(CmdBuf));
+			FreeStrBuf(&CmdBuf);
 
-		StrBuf_ServGetln(Buf);
-		if (GetServerStatus(Buf, NULL) == 4) {
-			if (save_to_drafts) {
-				if (  (havebstr("recp"))
-				    || (havebstr("cc"  ))
-				    || (havebstr("bcc" )) ) {
-					/* save recipient headers or room to post to */
-					serv_printf("To: %s", ChrPtr(Recp));
-					serv_printf("Cc: %s", ChrPtr(Cc));
-					serv_printf("Bcc: %s", ChrPtr(Bcc));
-				} else {
-					serv_printf("X-Citadel-Room: %s", ChrPtr(WC->wc_roomname));
+			StrBuf_ServGetln(Buf);
+			if (GetServerStatus(Buf, NULL) == 4) {
+				if (save_to_drafts) {
+					if (  (havebstr("recp"))
+					      || (havebstr("cc"  ))
+					      || (havebstr("bcc" )) ) {
+						/* save recipient headers or room to post to */
+						serv_printf("To: %s", ChrPtr(Recp));
+						serv_printf("Cc: %s", ChrPtr(Cc));
+						serv_printf("Bcc: %s", ChrPtr(Bcc));
+					} else {
+						serv_printf("X-Citadel-Room: %s", ChrPtr(WC->CurRoom.name));
+					}
 				}
-			}
-			post_mime_to_server();
-			if (save_to_drafts) {
-				StrBufAppendBufPlain(WCC->ImportantMsg, _("Message has been saved to Drafts.\n"), -1, 0);
-				gotoroom(WCC->wc_roomname);
+				post_mime_to_server();
+				if (save_to_drafts) {
+					StrBufAppendBufPlain(WCC->ImportantMsg, _("Message has been saved to Drafts.\n"), -1, 0);
+					gotoroom(WCC->CurRoom.name);
+					display_enter();
+					FreeStrBuf(&Buf);
+					return;
+				} else if (  (havebstr("recp"))
+					     || (havebstr("cc"  ))
+					     || (havebstr("bcc" ))
+					) {
+					StrBufAppendBufPlain(WCC->ImportantMsg, _("Message has been sent.\n"), -1, 0);
+				}
+				else {
+					StrBufAppendBufPlain(WCC->ImportantMsg, _("Message has been posted.\n"), -1, 0);
+				}
+				dont_post = lbstr("postseq");
+			} else {
+				StrBufCutLeft(Buf, 4);
+
+				lprintf(9, "%s:%d: server post error: %s\n", __FILE__, __LINE__, ChrPtr(Buf));
+				StrBufAppendBuf(WCC->ImportantMsg, Buf, 0);
+				if (save_to_drafts) gotoroom(WCC->CurRoom.name);
 				display_enter();
 				FreeStrBuf(&Buf);
+				FreeStrBuf(&Cc);
+				FreeStrBuf(&Bcc);
 				return;
-			} else if (  (havebstr("recp"))
-			   || (havebstr("cc"  ))
-			   || (havebstr("bcc" ))
-			) {
-				StrBufAppendBufPlain(WCC->ImportantMsg, _("Message has been sent.\n"), -1, 0);
 			}
-			else {
-				StrBufAppendBufPlain(WCC->ImportantMsg, _("Message has been posted.\n"), -1, 0);
-			}
-			dont_post = lbstr("postseq");
-		} else {
-			StrBufCutLeft(Buf, 4);
-
-			lprintf(9, "%s:%d: server post error: %s\n", __FILE__, __LINE__, ChrPtr(Buf));
-			StrBufAppendBuf(WCC->ImportantMsg, Buf, 0);
-			if (save_to_drafts) gotoroom(WCC->wc_roomname);
-			display_enter();
-			FreeStrBuf(&Buf);
-			return;
 		}
 		FreeStrBuf(&Buf);
+		FreeStrBuf(&Cc);
+		FreeStrBuf(&Bcc);
 	}
 
 	DeleteHash(&WCC->attachments);
@@ -1242,7 +1258,7 @@ void post_message(void)
 	 *  Otherwise, just go to the "read messages" loop.
 	 */
 	else {
-		readloop(readnew);
+		readloop(readnew, eUseDefault);
 	}
 }
 
@@ -1284,7 +1300,7 @@ void display_enter(void)
 	}
 	else if (buf[0] != '2') {		/* Any other error means that we cannot continue */
 		sprintf(WCC->ImportantMessage, "%s", &buf[4]);
-		readloop(readnew);
+		readloop(readnew, eUseDefault);
 		return;
 	}
 
@@ -1297,8 +1313,8 @@ void display_enter(void)
 	 * Are we perhaps in an address book view?  If so, then an "enter
 	 * message" command really means "add new entry."
 	 */
-	if (WCC->wc_default_view == VIEW_ADDRESSBOOK) {
-		do_edit_vcard(-1, "", NULL, NULL, "",  ChrPtr(WCC->wc_roomname));
+	if (WCC->CurRoom.defview == VIEW_ADDRESSBOOK) {
+		do_edit_vcard(-1, "", NULL, NULL, "",  ChrPtr(WCC->CurRoom.name));
 		return;
 	}
 
@@ -1306,7 +1322,7 @@ void display_enter(void)
 	 * Are we perhaps in a calendar room?  If so, then an "enter
 	 * message" command really means "add new calendar item."
 	 */
-	if (WCC->wc_default_view == VIEW_CALENDAR) {
+	if (WCC->CurRoom.defview == VIEW_CALENDAR) {
 		display_edit_event();
 		return;
 	}
@@ -1315,7 +1331,7 @@ void display_enter(void)
 	 * Are we perhaps in a tasks view?  If so, then an "enter
 	 * message" command really means "add new task."
 	 */
-	if (WCC->wc_default_view == VIEW_TASKS) {
+	if (WCC->CurRoom.defview == VIEW_TASKS) {
 		display_edit_task();
 		return;
 	}
@@ -1391,7 +1407,7 @@ void delete_msg(void)
 
 	msgid = lbstr("msgid");
 
-	if (WC->wc_is_trash) {	/* Delete from Trash is a real delete */
+	if ((WC->CurRoom.RAFlags & UA_ISTRASH) != 0) {	/* Delete from Trash is a real delete */
 		serv_printf("DELE %ld", msgid);	
 	}
 	else {			/* Otherwise move it to Trash */
@@ -1400,7 +1416,7 @@ void delete_msg(void)
 
 	serv_getln(buf, sizeof buf);
 	sprintf(WC->ImportantMessage, "%s", &buf[4]);
-	readloop(readnew);
+	readloop(readnew, eUseDefault);
 }
 
 
@@ -1423,7 +1439,7 @@ void move_msg(void)
 		sprintf(WC->ImportantMessage, (_("The message was not moved.")));
 	}
 
-	readloop(readnew);
+	readloop(readnew, eUseDefault);
 }
 
 
@@ -1494,8 +1510,10 @@ void postpart(StrBuf *partnum, StrBuf *filename, int force_download)
 	void *vPart;
 	StrBuf *content_type;
 	wc_mime_attachment *part;
-	
-	if (GetHash(WC->attachments, SKEY(partnum), &vPart) &&
+	int i;
+
+	i = StrToi(partnum);
+	if (GetHash(WC->attachments, IKEY(i), &vPart) &&
 	    (vPart != NULL)) {
 		part = (wc_mime_attachment*) vPart;
 		if (force_download) {
@@ -1550,7 +1568,7 @@ void mimepart(int force_download)
 			StrBufExtract_token(ContentType, Buf, 3, '|');
 		}
 
-		read_server_binary(WCC->WBuf, bytes, Buf);
+		serv_read_binary(WCC->WBuf, bytes, Buf);
 		serv_puts("CLOS");
 		StrBuf_ServGetln(Buf);
 		CT = ChrPtr(ContentType);
@@ -1611,14 +1629,20 @@ StrBuf *load_mimepart(long msgnum, char *partnum)
 void MimeLoadData(wc_mime_attachment *Mime)
 {
 	StrBuf *Buf;
+	const char *Ptr;
 	off_t bytes;
 	/* TODO: is there a chance the content type is different from the one we know? */
+
 	serv_printf("DLAT %ld|%s", Mime->msgnum, ChrPtr(Mime->PartNum));
 	Buf = NewStrBuf();
 	StrBuf_ServGetln(Buf);
 	if (GetServerStatus(Buf, NULL) == 6) {
-		bytes = extract_long(&(ChrPtr(Buf)[4]), 0);
-				     
+		Ptr = &(ChrPtr(Buf)[4]);
+		bytes = StrBufExtractNext_long(Buf, &Ptr, '|');
+		StrBufSkip_NTokenS(Buf, &Ptr, '|', 3);  /* filename, cbtype, mimetype */
+		if (Mime->Charset == NULL) Mime->Charset = NewStrBuf();
+		StrBufExtract_NextToken(Mime->Charset, Buf, &Ptr, '|');
+		
 		if (Mime->Data == NULL)
 			Mime->Data = NewStrBufPlain(NULL, bytes);
 		StrBuf_ServGetBLOBBuffered(Mime->Data, bytes);
@@ -1667,12 +1691,13 @@ void download_postpart(void) {
 	FreeStrBuf(&partnum);
 }
 
-void h_readnew(void) { readloop(readnew);}
-void h_readold(void) { readloop(readold);}
-void h_readfwd(void) { readloop(readfwd);}
-void h_headers(void) { readloop(headers);}
-void h_do_search(void) { readloop(do_search);}
-void h_readgt(void) { readloop(readgt);}
+void h_readnew(void) { readloop(readnew, eUseDefault);}
+void h_readold(void) { readloop(readold, eUseDefault);}
+void h_readfwd(void) { readloop(readfwd, eUseDefault);}
+void h_headers(void) { readloop(headers, eUseDefault);}
+void h_do_search(void) { readloop(do_search, eUseDefault);}
+void h_readgt(void) { readloop(readgt, eUseDefault);}
+void h_readlt(void) { readloop(readlt, eUseDefault);}
 
 void jsonMessageListHdr(void) 
 {
@@ -1692,7 +1717,7 @@ void jsonMessageList(void) {
 	long oper = (havebstr("query")) ? do_search : readnew;
 	WC->is_ajax = 1; 
 	gotoroom(room);
-	readloop(oper);
+	readloop(oper, eUseDefault);
 	WC->is_ajax = 0;
 }
 
@@ -1700,6 +1725,7 @@ void RegisterReadLoopHandlerset(
 	int RoomType,
 	GetParamsGetServerCall_func GetParamsGetServerCall,
 	PrintViewHeader_func PrintViewHeader,
+	load_msg_ptrs_detailheaders LH,
 	LoadMsgFromServer_func LoadMsgFromServer,
 	RenderView_or_Tail_func RenderView_or_Tail,
 	View_Cleanup_func ViewCleanup
@@ -1715,6 +1741,7 @@ void RegisterReadLoopHandlerset(
 	Handler->LoadMsgFromServer = LoadMsgFromServer;
 	Handler->RenderView_or_Tail = RenderView_or_Tail;
 	Handler->ViewCleanup = ViewCleanup;
+	Handler->LHParse = LH;
 
 	Put(ReadLoopHandler, IKEY(RoomType), Handler, NULL);
 }
@@ -1748,12 +1775,13 @@ InitModule_MSG
 	WebcitAddUrlHandler(HKEY("readfwd"), "", 0, h_readfwd, NEED_URL);
 	WebcitAddUrlHandler(HKEY("headers"), "", 0, h_headers, NEED_URL);
 	WebcitAddUrlHandler(HKEY("readgt"), "", 0, h_readgt, NEED_URL);
+	WebcitAddUrlHandler(HKEY("readlt"), "", 0, h_readlt, NEED_URL);
 	WebcitAddUrlHandler(HKEY("do_search"), "", 0, h_do_search, 0);
 	WebcitAddUrlHandler(HKEY("display_enter"), "", 0, display_enter, 0);
-	WebcitAddUrlHandler(HKEY("post"), "", 0, post_message, 0);
-	WebcitAddUrlHandler(HKEY("move_msg"), "", 0, move_msg, 0);
-	WebcitAddUrlHandler(HKEY("delete_msg"), "", 0, delete_msg, 0);
-	WebcitAddUrlHandler(HKEY("confirm_move_msg"), "", 0, confirm_move_msg, 0);
+	WebcitAddUrlHandler(HKEY("post"), "", 0, post_message, PROHIBIT_STARTPAGE);
+	WebcitAddUrlHandler(HKEY("move_msg"), "", 0, move_msg, PROHIBIT_STARTPAGE);
+	WebcitAddUrlHandler(HKEY("delete_msg"), "", 0, delete_msg, PROHIBIT_STARTPAGE);
+	WebcitAddUrlHandler(HKEY("confirm_move_msg"), "", 0, confirm_move_msg, PROHIBIT_STARTPAGE);
 	WebcitAddUrlHandler(HKEY("msg"), "", 0, embed_message, NEED_URL);
 	WebcitAddUrlHandler(HKEY("message"), "", 0, handle_one_message, NEED_URL|XHTTP_COMMANDS|COOKIEUNNEEDED|FORCE_SESSIONCLOSE);
 	WebcitAddUrlHandler(HKEY("printmsg"), "", 0, print_message, NEED_URL);
@@ -1762,8 +1790,8 @@ InitModule_MSG
 
 	WebcitAddUrlHandler(HKEY("mimepart"), "", 0, view_mimepart, NEED_URL);
 	WebcitAddUrlHandler(HKEY("mimepart_download"), "", 0, download_mimepart, NEED_URL);
-	WebcitAddUrlHandler(HKEY("postpart"), "", 0, view_postpart, NEED_URL);
-	WebcitAddUrlHandler(HKEY("postpart_download"), "", 0, download_postpart, NEED_URL);
+	WebcitAddUrlHandler(HKEY("postpart"), "", 0, view_postpart, NEED_URL|PROHIBIT_STARTPAGE);
+	WebcitAddUrlHandler(HKEY("postpart_download"), "", 0, download_postpart, NEED_URL|PROHIBIT_STARTPAGE);
 
 	/* json */
 	WebcitAddUrlHandler(HKEY("roommsgs"), "", 0, jsonMessageList,0);
