@@ -1,5 +1,5 @@
 /*
- * $Id: notes.c 7939 2009-10-23 12:51:53Z dothebart $
+ * $Id: notes.c 8255 2010-01-24 21:59:37Z dothebart $
  *
  */
 
@@ -351,7 +351,7 @@ void add_new_note(void) {
 		vnote_free(v);
 	}
 	
-	readloop(readfwd);
+	readloop(readfwd, eUseDefault);
 }
 
 
@@ -446,6 +446,7 @@ InitModule_NOTES
 		VIEW_NOTES,
 		notes_GetParamsGetServerCall,
 		NULL,
+		NULL,
 		notes_LoadMsgFromServer,
 		NULL,
 		notes_Cleanup);
