@@ -1,5 +1,5 @@
 /*
- * $Id: preferences.c 7960 2009-10-25 09:33:23Z dothebart $
+ * $Id: preferences.c 8668 2010-07-08 18:55:13Z dothebart $
  *
  * Manage user preferences with a little help from the Citadel server.
  *
@@ -10,9 +10,10 @@
 #include "groupdav.h"
 
 HashList *PreferenceHooks;
+extern HashList *HandlerHash;
 
 typedef struct _PrefDef {
-	long Type;
+	ePrefType eType;
 	StrBuf *Setting;
 	const char *PrefStr;
 	PrefEvalFunc OnLoad;
@@ -20,9 +21,11 @@ typedef struct _PrefDef {
 } PrefDef;
 
 typedef struct _Preference {
+	PrefDef *Type;
+	ePrefType eFlatPrefType;
+
 	StrBuf *Key;
 	StrBuf *Val;
-	PrefDef *Type;
 
 	long lval;
 	long decoded;
@@ -48,14 +51,14 @@ void DestroyPreference(void *vPref)
 }
 void _RegisterPreference(const char *Setting, long SettingLen, 
 			 const char *PrefStr, 
-			 long Type, 
+			 ePrefType Type, 
 			 PrefEvalFunc OnLoad, 
 			 const char *OnLoadName)
 {
 	PrefDef *Newpref = (PrefDef*) malloc(sizeof(PrefDef));
 	Newpref->Setting = NewStrBufPlain(Setting, SettingLen);
 	Newpref->PrefStr = PrefStr;
-	Newpref->Type = Type;
+	Newpref->eType = Type;
 	Newpref->OnLoad = OnLoad;
 	if (Newpref->OnLoad != NULL) {
 		Newpref->OnLoadName = NewStrBufPlain(OnLoadName, -1);
@@ -106,15 +109,17 @@ void GetPrefTypes(HashList *List)
 		{
 			PrefType = (PrefDef*) vPrefDef;
 			Pref->Type = PrefType;
+			Pref->eFlatPrefType = Pref->Type->eType;
 
 			lprintf(1, "Loading [%s]with type [%ld] [\"%s\"]\n",
 				ChrPtr(Pref->Key),
-				Pref->Type->Type,
+				Pref->Type->eType,
 				ChrPtr(Pref->Val));
 
-			switch (Pref->Type->Type)
+			switch (Pref->Type->eType)
 			{
-
+			case PRF_UNSET: /* WHUT? */
+				break;
 			case PRF_STRING:
 				break;
 			case PRF_INT:
@@ -197,14 +202,18 @@ void ParsePref(HashList **List, StrBuf *ReadBuf)
  */
 void load_preferences(void) 
 {
+	folder Room;
 	wcsession *WCC = WC;
 	int Done = 0;
 	StrBuf *ReadBuf;
 	long msgnum = 0L;
 	
+	memset(&Room, 0, sizeof(folder));
 	ReadBuf = NewStrBufPlain(NULL, SIZ * 4);
-	if (goto_config_room(ReadBuf) != 0) {
+	if (goto_config_room(ReadBuf, &Room) != 0) {
 		FreeStrBuf(&ReadBuf);
+		FlushFolder(&Room);
+
 		return;	/* oh well. */
 	}
 
@@ -239,19 +248,20 @@ void load_preferences(void)
 	}
 
 	/* Go back to the room we're supposed to be in */
-	if (StrLength(WCC->wc_roomname) > 0) {
-		serv_printf("GOTO %s", ChrPtr(WCC->wc_roomname));
+	if (StrLength(WCC->CurRoom.name) > 0) {
+		serv_printf("GOTO %s", ChrPtr(WCC->CurRoom.name));
 		StrBuf_ServGetln(ReadBuf);
 		GetServerStatus(ReadBuf, NULL);
 	}
 	FreeStrBuf(&ReadBuf);
+	FlushFolder(&Room);
 }
 
 /*
  * Goto the user's configuration room, creating it if necessary.
  * returns 0 on success or nonzero upon failure.
  */
-int goto_config_room(StrBuf *Buf) 
+int goto_config_room(StrBuf *Buf, folder *Room) 
 {
 	serv_printf("GOTO %s", USERCONFIGROOM);
 	StrBuf_ServGetln(Buf);
@@ -266,6 +276,7 @@ int goto_config_room(StrBuf *Buf)
 			return(1);
 		}
 	}
+	ParseGoto(Room, Buf);
 	return(0);
 }
 
@@ -330,16 +341,36 @@ void WritePrefsToServer(HashList *Hash)
  */
 void save_preferences(void) 
 {
+	folder Room;
 	wcsession *WCC = WC;
 	int Done = 0;
 	StrBuf *ReadBuf;
 	long msgnum = 0L;
 	
 	ReadBuf = NewStrBuf();
-	if (goto_config_room(ReadBuf) != 0) {
+	memset(&Room, 0, sizeof(folder));
+	if (goto_config_room(ReadBuf, &Room) != 0) {
 		FreeStrBuf(&ReadBuf);
+		FlushFolder(&Room);
+
 		return;	/* oh well. */
 	}
+
+	/* make shure the config room has the right type, else it might reject our config */
+	if (Room.view != VIEW_BBS) {
+		serv_printf("VIEW %d", VIEW_BBS);
+		StrBuf_ServGetln(ReadBuf);
+		if (GetServerStatus(ReadBuf, NULL) != 2) {
+			/* UPS? */
+		}
+		else if (goto_config_room(ReadBuf, &Room) != 0) {
+			FreeStrBuf(&ReadBuf);
+			FlushFolder(&Room);
+			
+			return;	/* oh well. */
+		}
+	}
+
 	serv_puts("MSGS ALL|0|1");
 	StrBuf_ServGetln(ReadBuf);
 	if (GetServerStatus(ReadBuf, NULL) == 8) {
@@ -372,12 +403,13 @@ void save_preferences(void)
 	}
 
 	/** Go back to the room we're supposed to be in */
-	if (StrLength(WCC->wc_roomname) > 0) {
-		serv_printf("GOTO %s", ChrPtr(WCC->wc_roomname));
+	if (StrLength(WCC->CurRoom.name) > 0) {
+		serv_printf("GOTO %s", ChrPtr(WCC->CurRoom.name));
 		StrBuf_ServGetln(ReadBuf);
 		GetServerStatus(ReadBuf, NULL);
 	}
 	FreeStrBuf(&ReadBuf);
+	FlushFolder(&Room);
 }
 
 /**
@@ -416,6 +448,63 @@ int get_PREFERENCE(const char *key, size_t keylen, StrBuf **value)
 	return Ret;
 }
 
+/**
+ * \brief	Write a key into the webcit preferences database for this user
+ *
+ * \params	key		key whichs value is to be modified
+ * \param keylen length of the key string
+ * \param	value		value to set
+ * \param	save_to_server	1 = flush all data to the server, 0 = cache it for now
+ */
+long compare_preference(const Preference *PrefA, 
+			const Preference *PrefB)
+{
+	ePrefType TypeA, TypeB;
+
+	if (PrefA->Type != NULL)
+		TypeA = PrefA->Type->eType;
+	else 
+		TypeA = PrefA->eFlatPrefType;
+
+	if (PrefB->Type != NULL)
+		TypeB = PrefB->Type->eType;
+	else 
+		TypeB = PrefB->eFlatPrefType;
+
+	if ((TypeA != PRF_UNSET) && 
+	    (TypeB != PRF_UNSET) && 
+	    (TypeA != TypeB))
+	{
+		if (TypeA > TypeB)
+			return 1;
+		else /* (PrefA->Type < PrefB->Type) */
+			return -1;
+	}
+
+	if (TypeB == PRF_UNSET)
+		TypeA = PRF_UNSET;
+		    
+	switch (TypeA)
+	{
+	default:
+	case PRF_UNSET:
+	case PRF_STRING:
+		return strcmp(ChrPtr(PrefA->Val), 
+			      ChrPtr(PrefB->Val));
+	case PRF_YESNO:
+	case PRF_INT:
+		if (PrefA->lval == PrefB->lval)
+			return 0;
+		else if (PrefA->lval > PrefB->lval)
+			return 1;
+		else
+			return -1;
+	case PRF_QP_STRING:
+		return strcmp(ChrPtr(PrefA->DeQPed), 
+			      ChrPtr(PrefB->DeQPed));
+	}
+}
+
 /**
  * \brief	Write a key into the webcit preferences database for this user
  *
@@ -433,6 +522,7 @@ void set_preference_backend(const char *key, size_t keylen,
 {
 	wcsession *WCC = WC;
 	void *vPrefDef;
+	void *vPrefB;
 	Preference *Pref;
 
 	Pref = (Preference*) malloc(sizeof(Preference));
@@ -447,11 +537,13 @@ void set_preference_backend(const char *key, size_t keylen,
 	if (PrefType != NULL)
 	{
 		Pref->Type = PrefType;
-		if (Pref->Type->Type != lPrefType)
+		Pref->eFlatPrefType = PrefType->eType;
+		if (Pref->Type->eType != lPrefType)
 			lprintf(1, "warning: saving preference with wrong type [%s] %ld != %ld \n",
-				key, Pref->Type->Type, lPrefType);
-		switch (Pref->Type->Type)
+				key, Pref->Type->eType, lPrefType);
+		switch (Pref->Type->eType)
 		{
+		case PRF_UNSET: /* default to string... */
 		case PRF_STRING:
 			Pref->Val = value;
 			Pref->decoded = 1;
@@ -483,6 +575,7 @@ void set_preference_backend(const char *key, size_t keylen,
 			Pref->Type->OnLoad(Pref->Val, Pref->lval);
 	}
 	else {
+		Pref->eFlatPrefType = lPrefType;
 		switch (lPrefType)
 		{
 		case PRF_STRING:
@@ -513,6 +606,13 @@ void set_preference_backend(const char *key, size_t keylen,
 			break;
 		}
 	}
+
+	if ((save_to_server != 0) && 
+	    GetHash(WCC->hash_prefs, key, keylen, &vPrefB) && 
+	    (vPrefB != NULL) && 
+	    (compare_preference (Pref, vPrefB) == 0))
+		save_to_server = 0;
+
 	Put(WCC->hash_prefs, key, keylen, Pref, DestroyPreference);
 	
 	if (save_to_server) WCC->SavePrefsToServer = 1;
@@ -581,7 +681,7 @@ int get_room_prefs_backend(const char *key, size_t keylen,
 	int Ret;
 
 	pref_name = NewStrBufPlain (HKEY("ROOM:"));
-	StrBufAppendBuf(pref_name, WC->wc_roomname, 0);
+	StrBufAppendBuf(pref_name, WC->CurRoom.name, 0);
 	StrBufAppendBufPlain(pref_name, HKEY(":"), 0);
 	StrBufAppendBufPlain(pref_name, key, keylen, 0);
 	Ret = get_pref_backend(SKEY(pref_name), Pref);
@@ -665,7 +765,7 @@ void set_ROOM_PREFS(const char *key, size_t keylen, StrBuf *value, int save_to_s
 	StrBuf *pref_name;
 	
 	pref_name = NewStrBufPlain (HKEY("ROOM:"));
-	StrBufAppendBuf(pref_name, WC->wc_roomname, 0);
+	StrBufAppendBuf(pref_name, WC->CurRoom.name, 0);
 	StrBufAppendBufPlain(pref_name, HKEY(":"), 0);
 	StrBufAppendBufPlain(pref_name, key, keylen, 0);
 	set_preference_backend(SKEY(pref_name), 0, value, PRF_STRING, save_to_server, NULL);
@@ -694,7 +794,8 @@ void GetPreferences(HashList *Setting)
 
 		if (!HaveBstr(SKEY(PrefType->Setting)))
 			continue;
-		switch (PrefType->Type) {
+		switch (PrefType->eType) {
+		case PRF_UNSET:
 		case PRF_STRING:
 			Buf = NewStrBufDup(SBstr(SKEY(PrefType->Setting)));
 			set_preference_backend(SKEY(PrefType->Setting),
@@ -763,8 +864,9 @@ void tmplput_CFG_Value(StrBuf *Target, WCTemplputParams *TP)
 		if (Pref->Type == NULL) {
 			StrBufAppendTemplate(Target, TP, Pref->Val, 1);
 		}
-		switch (Pref->Type->Type)
+		switch (Pref->Type->eType)
 		{
+		case PRF_UNSET: /* default to string... */
 		case PRF_STRING:
 			StrBufAppendTemplate(Target, TP, Pref->Val, 1);
 			break;
@@ -840,11 +942,20 @@ int ConditionalPreference(StrBuf *Target, WCTemplputParams *TP)
 	if (!HAVE_PARAM(3)) {
 		return 1;
 	}
-	else if (TP->Tokens->Params[3]->Type == TYPE_STR)
-		return ((TP->Tokens->Params[3]->len == StrLength(Pref)) &&
-			(strcmp(TP->Tokens->Params[3]->Start, ChrPtr(Pref)) == 0));
+	else if (IS_NUMBER(TP->Tokens->Params[3]->Type))
+	{
+		return StrTol(Pref) == GetTemplateTokenNumber (Target, TP, 3, 0);
+	}
 	else 
-		return (StrTol(Pref) == TP->Tokens->Params[3]->lvalue);
+	{
+		const char *pch;
+		long len;
+		
+		GetTemplateTokenString(Target, TP, 3, &pch, &len);
+		
+		return ((len == StrLength(Pref)) &&
+			(strcmp(pch, ChrPtr(Pref)) == 0));
+	}
 }
 
 int ConditionalHasPreference(StrBuf *Target, WCTemplputParams *TP)
@@ -957,23 +1068,50 @@ void offer_start_page(StrBuf *Target, WCTemplputParams *TP)
  */
 void change_start_page(void) 
 {
-	if (!havebstr("startpage")) {
+	wcsession *WCC = WC;
+	const char *pch;
+	void *vHandler;
+	int ProhibitSave = 0;
+	const StrBuf *pStartPage = sbstr("startpage");
+
+	if (pStartPage != NULL) {
+		pch = strchr(ChrPtr(pStartPage), '?');
+
+		if ((pch != NULL) && (
+			    GetHash(HandlerHash, ChrPtr(pStartPage), pch - ChrPtr(pStartPage), &vHandler), 
+			    (vHandler != NULL) &&
+			    ((((WebcitHandler*)vHandler)->Flags & PROHIBIT_STARTPAGE) != 0)))
+		{ /* OK, This handler doesn't want to be set as start page, prune it. */
+			ProhibitSave = 1;
+		}
+	}
+
+	if ((pStartPage == NULL) || 
+	    (ProhibitSave == 1))
+	{
 		set_preference_backend(HKEY("startpage"), 
 				       0, 
 				       NewStrBufPlain(HKEY("")),
 				       PRF_STRING,
 				       1, 
 				       NULL);
-		safestrncpy(WC->ImportantMessage,
-			    _("You no longer have a start page selected."),
-			    sizeof( WC->ImportantMessage));
+		if (ProhibitSave == 1)
+			StrBufAppendBufPlain(WCC->ImportantMsg,
+					     _("This isn't allowed to become the start page."),
+					     -1, 0);
+		else
+			StrBufAppendBufPlain(WCC->ImportantMsg,
+					     _("You no longer have a start page selected."),
+					     -1, 0);
 		display_main_menu();
 		return;
 	}
 
+
+
 	set_preference_backend(HKEY("startpage"), 
 			       0, 
-			       NewStrBufDup(sbstr("startpage")),
+			       NewStrBufDup(pStartPage),
 			       PRF_STRING,
 			       1, 
 			       NULL);
@@ -987,6 +1125,7 @@ void change_start_page(void)
 void LoadStartpage(StrBuf *URL, long lvalue)
 {
 	const char *pch;
+	void *vHandler;
 	pch = strchr(ChrPtr(URL), '?');
 	if (pch == NULL) {
 		/* purge the sins of the past... */
@@ -996,6 +1135,13 @@ void LoadStartpage(StrBuf *URL, long lvalue)
 			WC->SavePrefsToServer = 1;
 		}
 	}
+	else if (GetHash(HandlerHash, ChrPtr(URL), pch - ChrPtr(URL), &vHandler), 
+		 (vHandler != NULL) &&
+		 ((((WebcitHandler*)vHandler)->Flags & PROHIBIT_STARTPAGE) != 0))
+	{ /* OK, This handler doesn't want to be set as start page, prune it. */
+		FlushStrBuf(URL);
+		WC->SavePrefsToServer = 1;
+	}
 }
 
 
