@@ -1,5 +1,5 @@
 /*
- * $Id: roomlist.c 7751 2009-08-28 21:13:28Z dothebart $
+ * $Id: roomlist.c 8514 2010-04-19 10:07:49Z dothebart $
  * room listings and filters.
  */
 
@@ -9,16 +9,16 @@
 
 void DeleteFloor(void *vFloor)
 {
-	floor *Floor;
-	Floor = (floor*) vFloor;
-	FreeStrBuf(&Floor->Name);
-	free(Floor);
+	Floor *pFloor;
+	pFloor = (Floor*) vFloor;
+	FreeStrBuf(&pFloor->Name);
+	free(pFloor);
 }
 
 int SortFloorsByNameOrder(const void *vfloor1, const void *vfloor2) 
 {
-	floor *f1 = (floor*) GetSearchPayload(vfloor1);
-	floor *f2 = (floor*) GetSearchPayload(vfloor2);
+	Floor *f1 = (Floor*) GetSearchPayload(vfloor1);
+	Floor *f2 = (Floor*) GetSearchPayload(vfloor2);
 	
 	/* prefer My floor over alpabetical sort */
 	if (f1->ID == VIRTUAL_MY_FLOOR)
@@ -35,8 +35,9 @@ HashList *GetFloorListHash(StrBuf *Target, WCTemplputParams *TP)
 	const char *Err;
 	StrBuf *Buf;
 	HashList *floors;
+	HashList *floorsbyname;
 	HashPos *it;
-	floor *Floor;
+	Floor *pFloor;
 	void *vFloor;
 	const char *Pos;
 	int i;
@@ -48,14 +49,16 @@ HashList *GetFloorListHash(StrBuf *Target, WCTemplputParams *TP)
 	if (WCC->Floors != NULL)
 		return WCC->Floors;
 	WCC->Floors = floors = NewHash(1, Flathash);
+	WCC->FloorsByName = floorsbyname = NewHash(1, NULL);
 	Buf = NewStrBuf();
 
-	Floor = malloc(sizeof(floor));
-	Floor->ID = VIRTUAL_MY_FLOOR;
-	Floor->Name = NewStrBufPlain(_("My Folders"), -1);
-	Floor->NRooms = 0;
+	pFloor = (Floor*) malloc(sizeof(Floor));
+	pFloor->ID = VIRTUAL_MY_FLOOR;
+	pFloor->Name = NewStrBufPlain(_("My Folders"), -1);
+	pFloor->NRooms = 0;
 	
-	Put(floors, IKEY(Floor->ID), Floor, DeleteFloor);
+	Put(floors, IKEY(pFloor->ID), pFloor, DeleteFloor);
+	Put(floorsbyname, SKEY(pFloor->Name), pFloor, reference_free_handler);
 
 	serv_puts("LFLR"); /* get floors */
 	StrBufTCP_read_line(Buf, &WC->serv_sock, 0, &Err); /* '100', we hope */
@@ -72,13 +75,14 @@ HashList *GetFloorListHash(StrBuf *Target, WCTemplputParams *TP)
 			
 				Pos = NULL;
 
-				Floor = malloc(sizeof(floor));
-				Floor->ID = StrBufExtractNext_int(Buf, &Pos, '|');
-				Floor->Name = NewStrBufPlain(NULL, StrLength(Buf));
-				StrBufExtract_NextToken(Floor->Name, Buf, &Pos, '|');
-				Floor->NRooms = StrBufExtractNext_long(Buf, &Pos, '|');
+				pFloor = (Floor*) malloc(sizeof(Floor));
+				pFloor->ID = StrBufExtractNext_int(Buf, &Pos, '|');
+				pFloor->Name = NewStrBufPlain(NULL, StrLength(Buf));
+				StrBufExtract_NextToken(pFloor->Name, Buf, &Pos, '|');
+				pFloor->NRooms = StrBufExtractNext_long(Buf, &Pos, '|');
 
-				Put(floors, IKEY(Floor->ID), Floor, DeleteFloor);
+				Put(floors, IKEY(pFloor->ID), pFloor, DeleteFloor);
+				Put(floorsbyname, SKEY(pFloor->Name), pFloor, reference_free_handler);
 			}
 	}
 	FreeStrBuf(&Buf);
@@ -88,7 +92,7 @@ HashList *GetFloorListHash(StrBuf *Target, WCTemplputParams *TP)
 	SortByPayload(floors, SortFloorsByNameOrder);
 	it = GetNewHashPos(floors, 0);
 	while (	GetNextHashPos(floors, it, &HKLen, &HashKey, &vFloor)) 
-		((floor*) vFloor)->AlphaN = i++;
+		((Floor*) vFloor)->AlphaN = i++;
 	DeleteHashPos(&it);
 	SortByHashKeyStr(floors);
 
@@ -97,23 +101,23 @@ HashList *GetFloorListHash(StrBuf *Target, WCTemplputParams *TP)
 
 void tmplput_FLOOR_ID(StrBuf *Target, WCTemplputParams *TP) 
 {
-	floor *Floor = (floor *)(TP->Context);
+	Floor *myFloor = (Floor *)CTX;
 
-	StrBufAppendPrintf(Target, "%d", Floor->ID);
+	StrBufAppendPrintf(Target, "%d", myFloor->ID);
 }
 
 void tmplput_FLOOR_NAME(StrBuf *Target, WCTemplputParams *TP) 
 {
-	floor *Floor = (floor *)(TP->Context);
+	Floor *myFloor = (Floor *)CTX;
 
-	StrBufAppendTemplate(Target, TP, Floor->Name, 0);
+	StrBufAppendTemplate(Target, TP, myFloor->Name, 0);
 }
 
 void tmplput_FLOOR_NROOMS(StrBuf *Target, WCTemplputParams *TP) 
 {
-	floor *Floor = (floor *)(TP->Context);
+	Floor *myFloor = (Floor *)CTX;
 
-	StrBufAppendPrintf(Target, "%d", Floor->NRooms);
+	StrBufAppendPrintf(Target, "%d", myFloor->NRooms);
 }
 HashList *GetRoomListHashLKRA(StrBuf *Target, WCTemplputParams *TP) 
 {
@@ -122,23 +126,37 @@ HashList *GetRoomListHashLKRA(StrBuf *Target, WCTemplputParams *TP)
 	if (WCC->Floors == NULL)
 		GetFloorListHash(Target, TP);
 	serv_puts("LKRA");
-	return GetRoomListHash(Target, TP);
+	if (WCC->Rooms == NULL) 
+		WCC->Rooms =  GetRoomListHash(Target, TP);
+	return WCC->Rooms;
 }
 
-void DeleteFolder(void *vFolder)
+void FlushFolder(folder *room)
 {
 	int i;
-	folder *room;
-	room = (folder*) vFolder;
 
 	FreeStrBuf(&room->name);
-
+	if (room->IgnetCfgs[0] == (HashList*) StrBufNOTNULL)
+	{
+		room->IgnetCfgs[0] = NULL;
+		for (i = ignet_push_share; i < maxRoomNetCfg; i++)
+			DeleteHash(&room->IgnetCfgs[i]);
+	}
 	if (room->RoomNameParts != NULL)
 	{
 		for (i=0; i < room->nRoomNameParts; i++)
 			FreeStrBuf(&room->RoomNameParts[i]);
 		free(room->RoomNameParts);
 	}
+}
+
+void vDeleteFolder(void *vFolder)
+{
+	folder *room;
+
+	room = (folder*) vFolder;
+	FlushFolder(room);
+
 	free(room);
 }
 
@@ -150,7 +168,6 @@ HashList *GetRoomListHash(StrBuf *Target, WCTemplputParams *TP)
 	folder *room;
 	StrBuf *Buf;
 	const char *Pos;
-	const char *Err;
 	void *vFloor;
 	wcsession *WCC = WC;
 	CompareFunc SortIt;
@@ -158,7 +175,7 @@ HashList *GetRoomListHash(StrBuf *Target, WCTemplputParams *TP)
 
 	Buf = NewStrBuf();
 	rooms = NewHash(1, NULL);
-	StrBufTCP_read_line(Buf, &WC->serv_sock, 0, &Err);
+	StrBuf_ServGetln(Buf);
 	if (GetServerStatus(Buf, NULL) == 1) 
 	{
 		while(!Done && StrBuf_ServGetln(Buf))
@@ -227,12 +244,12 @@ HashList *GetRoomListHash(StrBuf *Target, WCTemplputParams *TP)
 				}
 				/* get a pointer to the floor we're on: */
 				GetHash(WCC->Floors, IKEY(room->floorid), &vFloor);
-				room->Floor = (const floor*) vFloor;
+				room->Floor = (const Floor*) vFloor;
 
 
 
 				/* now we know everything, remember it... */
-				Put(rooms, SKEY(room->name), room, DeleteFolder);
+				Put(rooms, SKEY(room->name), room, vDeleteFolder);
 			}
 	}
 
@@ -246,6 +263,66 @@ HashList *GetRoomListHash(StrBuf *Target, WCTemplputParams *TP)
 	return rooms;
 }
 
+HashList *GetNetConfigHash(StrBuf *Target, WCTemplputParams *TP) 
+{
+	wcsession *WCC = WC;
+	StrBuf *Line;
+	StrBuf *Token;
+	StrBuf *Content;
+	long WantThisOne;
+	long PutTo;
+	long State;
+	
+	WantThisOne = GetTemplateTokenNumber(Target, TP, 5, 0);
+	if (WantThisOne == 0)
+		return NULL;
+	if (WCC->CurRoom.IgnetCfgs[0] == (HashList*) StrBufNOTNULL)
+		return WCC->CurRoom.IgnetCfgs[WantThisOne];
+
+	WCC->CurRoom.IgnetCfgs[0] = (HashList*) StrBufNOTNULL;
+	serv_puts("GNET");
+	Line = NewStrBuf();
+	Token = NewStrBuf();
+	StrBuf_ServGetln(Line);
+	if (GetServerStatus(Line, &State) == 1) 
+	{
+		const char *Pos = NULL;
+		StrBuf_ServGetln(Line);
+		StrBufExtract_NextToken(Token, Line, &Pos, '|');
+		PutTo = GetTokenDefine(SKEY(Token), -1);
+		if ((PutTo > 0) && 
+		    (PutTo < maxRoomNetCfg) &&
+		    (Pos != StrBufNOTNULL))
+		{
+			int n;
+			HashList *SubH;
+
+			if (WCC->CurRoom.IgnetCfgs[PutTo] == NULL)
+				WCC->CurRoom.IgnetCfgs[PutTo] = NewHash(1, NULL);
+			SubH = NewHash(1, NULL);
+			n = GetCount(WCC->CurRoom.IgnetCfgs[PutTo]) + 1;
+			Put(WCC->CurRoom.IgnetCfgs[PutTo], 
+			    IKEY(n),
+			    SubH, 
+			    HDeleteHash);
+			while (Pos != StrBufNOTNULL) {
+				Content = NewStrBuf();
+				StrBufExtract_NextToken(Content, Line, &Pos, '|');
+				Put(SubH, 
+				    IKEY(n),
+				    Content, 
+				    HFreeStrBuf);
+			}
+		}
+	}
+	else if (State == 550)
+		StrBufAppendBufPlain(WCC->ImportantMsg,
+				     _("Higher access is required to access this function."), -1, 0);
+
+
+	return WCC->CurRoom.IgnetCfgs[WantThisOne];
+}
+
 /** Unused function that orders rooms by the listorder flag */
 int SortRoomsByListOrder(const void *room1, const void *room2) 
 {
@@ -432,19 +509,24 @@ int GroupchangeRoomListByFloorRoomPrivFirst(const void *room1, const void *room2
 }
 
 
-
+int CompareRooms(const folder *room1, const folder *room2) 
+{
+	if ((room1 == NULL) || (room2 == NULL))
+		return -1;
+	return CompareRoomListByFloorRoomPrivFirst(room1, room2);
+}
 
 
 
 void tmplput_ROOM_NAME(StrBuf *Target, WCTemplputParams *TP) 
 {
-	folder *Folder = (folder *)(TP->Context);
+	folder *Folder = (folder *)CTX;
 
 	StrBufAppendTemplate(Target, TP, Folder->name, 0);
 }
 void tmplput_ROOM_BASENAME(StrBuf *Target, WCTemplputParams *TP) 
 {
-	folder *room = (folder *)(TP->Context);
+	folder *room = (folder *)CTX;
 
 	if (room->nRoomNameParts > 1)
 		StrBufAppendTemplate(Target, TP, 
@@ -454,7 +536,7 @@ void tmplput_ROOM_BASENAME(StrBuf *Target, WCTemplputParams *TP)
 }
 void tmplput_ROOM_LEVEL_N_TIMES(StrBuf *Target, WCTemplputParams *TP) 
 {
-	folder *room = (folder *)(TP->Context);
+	folder *room = (folder *)CTX;
 	int i;
         const char *AppendMe;
         long AppendMeLen;
@@ -470,7 +552,7 @@ void tmplput_ROOM_LEVEL_N_TIMES(StrBuf *Target, WCTemplputParams *TP)
 
 void tmplput_ROOM_ACL(StrBuf *Target, WCTemplputParams *TP) 
 {
-	folder *Folder = (folder *)(TP->Context);
+	folder *Folder = (folder *)CTX;
 
 	StrBufAppendPrintf(Target, "%ld", Folder->RAFlags, 0);
 }
@@ -478,7 +560,7 @@ void tmplput_ROOM_ACL(StrBuf *Target, WCTemplputParams *TP)
 
 void tmplput_ROOM_QRFLAGS(StrBuf *Target, WCTemplputParams *TP) 
 {
-	folder *Folder = (folder *)(TP->Context);
+	folder *Folder = (folder *)CTX;
 	StrBufAppendPrintf(Target, "%d", Folder->QRFlags);
 }
 
@@ -491,116 +573,116 @@ void tmplput_ROOM_RAFLAGS(StrBuf *Target, WCTemplputParams *TP)
 
 void tmplput_ROOM_FLOORID(StrBuf *Target, WCTemplputParams *TP) 
 {
-	folder *Folder = (folder *)(TP->Context);
+	folder *Folder = (folder *)CTX;
 	StrBufAppendPrintf(Target, "%d", Folder->floorid);
 }
 
 void tmplput_ROOM_LISTORDER(StrBuf *Target, WCTemplputParams *TP) 
 {
-	folder *Folder = (folder *)(TP->Context);
+	folder *Folder = (folder *)CTX;
 	StrBufAppendPrintf(Target, "%d", Folder->listorder);
 }
 void tmplput_ROOM_VIEW(StrBuf *Target, WCTemplputParams *TP) 
 {
-	folder *Folder = (folder *)(TP->Context);
+	folder *Folder = (folder *)CTX;
 	StrBufAppendPrintf(Target, "%d", Folder->view);
 }
 void tmplput_ROOM_DEFVIEW(StrBuf *Target, WCTemplputParams *TP) 
 {
-	folder *Folder = (folder *)(TP->Context);
+	folder *Folder = (folder *)CTX;
 	StrBufAppendPrintf(Target, "%d", Folder->defview);
 }
 void tmplput_ROOM_LASTCHANGE(StrBuf *Target, WCTemplputParams *TP) 
 {
-	folder *Folder = (folder *)(TP->Context);
+	folder *Folder = (folder *)CTX;
 	StrBufAppendPrintf(Target, "%d", Folder->lastchange);
 }
 void tmplput_ROOM_FLOOR_ID(StrBuf *Target, WCTemplputParams *TP) 
 {
-	folder *Folder = (folder *)(TP->Context);
-	const floor *Floor = Folder->Floor;
+	folder *Folder = (folder *)CTX;
+	const Floor *pFloor = Folder->Floor;
 
-	if (Floor == NULL)
+	if (pFloor == NULL)
 		return;
 
-	StrBufAppendPrintf(Target, "%d", Floor->ID);
+	StrBufAppendPrintf(Target, "%d", pFloor->ID);
 }
 
 void tmplput_ROOM_FLOOR_NAME(StrBuf *Target, WCTemplputParams *TP) 
 {
-	folder *Folder = (folder *)(TP->Context);
-	const floor *Floor = Folder->Floor;
+	folder *Folder = (folder *)CTX;
+	const Floor *pFloor = Folder->Floor;
 
-	if (Floor == NULL)
+	if (pFloor == NULL)
 		return;
 
-	StrBufAppendTemplate(Target, TP, Floor->Name, 0);
+	StrBufAppendTemplate(Target, TP, pFloor->Name, 0);
 }
 
 void tmplput_ROOM_FLOOR_NROOMS(StrBuf *Target, WCTemplputParams *TP) 
 {
-	folder *Folder = (folder *)(TP->Context);
-	const floor *Floor = Folder->Floor;
+	folder *Folder = (folder *)CTX;
+	const Floor *pFloor = Folder->Floor;
 
-	if (Floor == NULL)
+	if (pFloor == NULL)
 		return;
-	StrBufAppendPrintf(Target, "%d", Floor->NRooms);
+	StrBufAppendPrintf(Target, "%d", pFloor->NRooms);
 }
 
 
 
 int ConditionalRoomHas_UA_KNOWN(StrBuf *Target, WCTemplputParams *TP)
 {
-	folder *Folder = (folder *)(TP->Context);
+	folder *Folder = (folder *)CTX;
 	return (Folder->RAFlags & UA_KNOWN) != 0;
 }
 
 int ConditionalRoomHas_UA_GOTOALLOWED(StrBuf *Target, WCTemplputParams *TP)
 {
-	folder *Folder = (folder *)(TP->Context);
+	folder *Folder = (folder *)CTX;
 	return (Folder->RAFlags & UA_GOTOALLOWED) != 0;
 }
 
 int ConditionalRoomHas_UA_HASNEWMSGS(StrBuf *Target, WCTemplputParams *TP)
 {
-	folder *Folder = (folder *)(TP->Context);
+	folder *Folder = (folder *)CTX;
 	return (Folder->RAFlags & UA_HASNEWMSGS) != 0;
 }
 
 int ConditionalRoomHas_UA_ZAPPED(StrBuf *Target, WCTemplputParams *TP)
 {
-	folder *Folder = (folder *)(TP->Context);
+	folder *Folder = (folder *)CTX;
 	return (Folder->RAFlags & UA_ZAPPED) != 0;
 }
 
 int ConditionalRoomHas_UA_POSTALLOWED(StrBuf *Target, WCTemplputParams *TP)
 {
-	folder *Folder = (folder *)(TP->Context);
+	folder *Folder = (folder *)CTX;
 	return (Folder->RAFlags & UA_POSTALLOWED) != 0;
 }
 
 int ConditionalRoomHas_UA_ADMINALLOWED(StrBuf *Target, WCTemplputParams *TP)
 {
-	folder *Folder = (folder *)(TP->Context);
+	folder *Folder = (folder *)CTX;
 	return (Folder->RAFlags & UA_ADMINALLOWED) != 0;
 }
 
 int ConditionalRoomHas_UA_DELETEALLOWED(StrBuf *Target, WCTemplputParams *TP)
 {
-	folder *Folder = (folder *)(TP->Context);
+	folder *Folder = (folder *)CTX;
 	return (Folder->RAFlags & UA_DELETEALLOWED) != 0;
 }
 
 
 int ConditionalRoomIsInbox(StrBuf *Target, WCTemplputParams *TP)
 {
-	folder *Folder = (folder *)(TP->Context);
+	folder *Folder = (folder *)CTX;
 	return Folder->is_inbox;
 }
 
 void tmplput_ROOM_COLLECTIONTYPE(StrBuf *Target, WCTemplputParams *TP) 
 {
-	folder *Folder = (folder *)(TP->Context);
+	folder *Folder = (folder *)CTX;
 	
 	switch(Folder->view) {
 	case VIEW_CALENDAR:
@@ -618,6 +700,9 @@ void tmplput_ROOM_COLLECTIONTYPE(StrBuf *Target, WCTemplputParams *TP)
 	case VIEW_JOURNAL:
 		StrBufAppendBufPlain(Target, HKEY("vjournal"), 0);
 		break;
+	case VIEW_WIKI:
+		StrBufAppendBufPlain(Target, HKEY("wiki"), 0);
+		break;
 	}
 }
 
@@ -626,7 +711,9 @@ void tmplput_ROOM_COLLECTIONTYPE(StrBuf *Target, WCTemplputParams *TP)
 
 int ConditionalRoomHasGroupdavContent(StrBuf *Target, WCTemplputParams *TP)
 {
-	folder *Folder = (folder *)(TP->Context);
+	folder *Folder = (folder *)CTX;
+
+	lprintf(0, "-> %s: %ld\n", ChrPtr(Folder->name), Folder->view);
 
 	return ((Folder->view == VIEW_CALENDAR) || 
 		(Folder->view == VIEW_TASKS) || 
@@ -640,59 +727,103 @@ int ConditionalRoomHasGroupdavContent(StrBuf *Target, WCTemplputParams *TP)
 int ConditionalFloorIsRESTSubFloor(StrBuf *Target, WCTemplputParams *TP)
 {
 	wcsession  *WCC = WC;
+	Floor *MyFloor = (Floor *)CTX;
+	/** if we have dav_depth the client just wants the subfloors */
+	if ((WCC->Hdr->HR.dav_depth == 1) && 
+	    (GetCount(WCC->Directory) == 0))
+		return 1;
+	return WCC->CurrentFloor == MyFloor;
+}
 
-	/** If we have dav_depth the client just wants the _current_ room without subfloors */
-	if (WCC->Hdr->HR.dav_depth == 0)
-		return 0;
-	    
-	return 1;
+
+int ConditionalFloorIsSUBROOM(StrBuf *Target, WCTemplputParams *TP)
+{
+	wcsession  *WCC = WC;
+	Floor *MyFloor = (Floor *)CTX;
+
+	return WCC->CurRoom.floorid == MyFloor->ID;
 }
 
 
 int ConditionalRoomIsRESTSubRoom(StrBuf *Target, WCTemplputParams *TP)
 {
 	wcsession  *WCC = WC;
-	folder     *Folder = (folder *)(TP->Context);
+	folder     *Folder = (folder *)CTX;
 	HashPos    *it;
 	StrBuf     * Dir;
 	void       *vDir;
 	long        len;
         const char *Key;
-	int i;
-
+	int i, j, urlp;
+	int delta;
 
 
+	/* list only folders relative to the current floor... */
 	if (Folder->Floor != WCC->CurrentFloor)
 		return 0;
 
-	if (GetCount(WCC->Directory) != Folder->nRoomNameParts)
+	urlp = GetCount(WCC->Directory);
+	delta = Folder->nRoomNameParts - urlp + 1;
+
+	lprintf(0, "\n->%s: %ld - %ld ", ChrPtr(Folder->name), urlp, 
+		Folder->nRoomNameParts);
+	/* list only the floors which are in relation to the dav_depth header */
+	if (WCC->Hdr->HR.dav_depth != delta) {
+		lprintf(0, "1\n");
 		return 0;
+	}
+
 
 	it = GetNewHashPos(WCC->Directory, 0);
-	for (i = 0; i < Folder->nRoomNameParts; i++)
-	{
-		if (!GetNextHashPos(WCC->Directory, it, &len, &Key, &vDir) ||
+	/* Fast forward the floorname we checked above... */
+	GetNextHashPos(WCC->Directory, it, &len, &Key, &vDir);
+
+	if (Folder->nRoomNameParts > 1) {		
+		for (i = 0, j = 1; 
+		     (i > Folder->nRoomNameParts) && (j > urlp); 
+		     i++, j++)
+		{
+			if (!GetNextHashPos(WCC->Directory, 
+					    it, &len, &Key, &vDir) ||
+			    (vDir == NULL))
+			{
+				DeleteHashPos(&it);
+
+				lprintf(0, "3\n");
+				return 0;
+			}
+			Dir = (StrBuf*) vDir;
+			if (strcmp(ChrPtr(Folder->RoomNameParts[i]), 
+				   ChrPtr(Dir)) != 0)
+			{
+				DeleteHashPos(&it);
+				lprintf(0, "4\n");
+				return 0;
+			}
+		}
+		DeleteHashPos(&it);
+		return 1;
+	}
+	else {
+		if (!GetNextHashPos(WCC->Directory, 
+				    it, &len, &Key, &vDir) ||
 		    (vDir == NULL))
 		{
 			DeleteHashPos(&it);
-			return 0;
+			
+			lprintf(0, "5\n");
+			return WCC->Hdr->HR.dav_depth == 1;
 		}
+		DeleteHashPos(&it);
 		Dir = (StrBuf*) vDir;
-		if (strcmp(ChrPtr(Folder->RoomNameParts[i]), 
-			   ChrPtr(Dir)) != 0)
-		{
-			DeleteHashPos(&it);
-			return 0;
+		if (WCC->Hdr->HR.dav_depth == 0) {
+			return (strcmp(ChrPtr(Folder->name), 
+				       ChrPtr(Dir))
+				== 0);
+
 		}
-	}
-	DeleteHashPos(&it);
-
-	/** If we have dav_depth the client just wants the _current_ room without subfloors */
-	if ((WCC->Hdr->HR.dav_depth == 0) &&
-	    (i != Folder->nRoomNameParts))
 		return 0;
-
-	return 1;
+	}
 }
 
 
@@ -709,29 +840,49 @@ void jsonRoomFlr(void)
 	end_burst(); 
 }
 
+
 void 
 SessionDetachModule_ROOMLIST
 (wcsession *sess)
 {
 	DeleteHash(&sess->Floors);
+	DeleteHash(&sess->Rooms);
+	DeleteHash(&sess->FloorsByName);
 }
 
 
+#define POP3_HOST 0
+#define POP3_USERNAME 1
+#define POP3_PASSWORD 2
+#define POP3_KEEP 3
+#define POP3_INTERVAL 4
+
 void 
 InitModule_ROOMLIST
 (void)
 {
+
+	REGISTERTokenParamDefine(POP3_HOST);
+	REGISTERTokenParamDefine(POP3_USERNAME);
+	REGISTERTokenParamDefine(POP3_PASSWORD);
+	REGISTERTokenParamDefine(POP3_KEEP);
+	REGISTERTokenParamDefine(POP3_INTERVAL);
+
+
 	WebcitAddUrlHandler(HKEY("json_roomflr"), "", 0, jsonRoomFlr, 0);
 
 
 	RegisterNamespace("FLOOR:ID", 0, 0, tmplput_FLOOR_ID, NULL, CTX_FLOORS);
 	RegisterNamespace("FLOOR:NAME", 0, 1, tmplput_FLOOR_NAME, NULL, CTX_FLOORS);
 	RegisterNamespace("FLOOR:NROOMS", 0, 0, tmplput_FLOOR_NROOMS, NULL, CTX_FLOORS);
+	RegisterConditional(HKEY("COND:FLOOR:ISSUBROOM"), 0, ConditionalFloorIsSUBROOM, CTX_FLOORS);
 	RegisterConditional(HKEY("COND:ROOM:REST:ISSUBFLOOR"), 0, ConditionalFloorIsRESTSubFloor, CTX_FLOORS);
 
+	RegisterIterator("ITERATE:THISROOM:GNET", 1, NULL, GetNetConfigHash, NULL, NULL, CTX_STRBUFARR, CTX_NONE, IT_NOFLAG);
+
 	RegisterIterator("LFLR", 0, NULL, GetFloorListHash, NULL, NULL, CTX_FLOORS, CTX_NONE, IT_FLAG_DETECT_GROUPCHANGE);
 
-	RegisterIterator("LKRA", 0, NULL, GetRoomListHashLKRA, NULL, DeleteHash, CTX_ROOMS, CTX_NONE, IT_FLAG_DETECT_GROUPCHANGE);
+	RegisterIterator("LKRA", 0, NULL, GetRoomListHashLKRA, NULL, NULL, CTX_ROOMS, CTX_NONE, IT_FLAG_DETECT_GROUPCHANGE);
 
 	RegisterNamespace("ROOM:INFO:FLOORID", 0, 1, tmplput_ROOM_FLOORID, NULL, CTX_ROOMS);
 	RegisterNamespace("ROOM:INFO:NAME", 0, 1, tmplput_ROOM_NAME, NULL, CTX_ROOMS);
