@@ -188,7 +188,7 @@ function resortAndDisplay(sortMode) {
 	var start = new Date();
 	var length = msgs.length;
 	var compiled = new Array(length+2);
-	compiled[0] = "<tbody xmlns:citadel=\"http://citadel.org\" id=\"message_list_body\" class=\"mailbox_summary\">";
+	compiled[0] = "<table class=\"mailbox_summary\" id=\"summary_headers\" \"cellspacing=0\" style=\"width:100%;-moz-user-select:none;\">";
 	for(var x=0; x<length; ++x) {
 		try {
 			var currentRow = msgs[x];
@@ -214,7 +214,7 @@ function resortAndDisplay(sortMode) {
 			WCLog("Exception on row " +  x + ":" + e);
 		}
 	}
-	compiled[length+2] = "</tbody>";
+	compiled[length+2] = "</table>";
 	var end = new Date();
 	WCLog("iterate: " + (end-start));
 	var compile = compiled.join("");
@@ -519,7 +519,8 @@ function ApplySorterToggle() {
 	}
 	currentSorterToggle.className = className;
 }
-/** Hack to make the header table line up with the data */
+
+/* Hack to make the header table line up with the data */
 function normalizeHeaderTable() {
 	var message_list_hdr = document.getElementById("message_list_hdr");
 	var summary_view = document.getElementById("summary_view");
