@@ -1,5 +1,5 @@
 /*
- * $Id: wclib.js 8163 2009-12-29 13:37:42Z dothebart $
+ * $Id: wclib.js 8648 2010-06-23 19:33:51Z ajc $
  * Copyright 2005 - 2009 The Citadel Team
  * Licensed under the GPL V3
  *
@@ -23,7 +23,6 @@ var supportsAddEventListener = (!!document.addEventListener);
 var today = new Date();
 
 var wc_log = "";
-var is_ie6 = false;
 if (document.all) {browserType = "ie"}
 if (window.navigator.userAgent.toLowerCase().match("gecko")) {
 	browserType= "gecko";
@@ -31,12 +30,8 @@ if (window.navigator.userAgent.toLowerCase().match("gecko")) {
 var ns6=document.getElementById&&!document.all;
 Event.observe(window, 'load', ToggleTaskDateOrNoDateActivate);
 Event.observe(window, 'load', taskViewActivate);
-Event.observe(window, 'load', fixbanner);
-Event.observe(window, 'load', resizeViewport);
-Event.observe(window, 'resize', resizeViewport);
 //document.observe("dom:loaded", setupPrefEngine);
 document.observe("dom:loaded", setupIconBar);
-document.observe('dom:loaded', function() { if (!!document.getElementById("ib_chat_launch")) { $('ib_chat_launch').observe('click', launchChat); } });
 function CtdlRandomString()  {
 	return((Math.random()+'').substr(3));
 }
@@ -171,7 +166,7 @@ function setupIconBar() {
   }
   var online_users = document.getElementById("online_users");
   if (online_users.offsetParent != null && online_users.offsetTop > 0) {
-    new Ajax.PeriodicalUpdater('online_users', 'do_template?template=wholist_section', {method: 'get', frequency: 30});
+    new Ajax.PeriodicalUpdater('online_users', 'do_template?template=who_iconbar', {method: 'get', frequency: 30});
   }
 }
 function changeIconBarEvent(event) {
@@ -853,92 +848,6 @@ function WCLog(msg) {
   }
 }
 
-function fixMissingCSSTable(elems) {
- if (elems[0] == null || elems[1] == null) {
-    return;
-  }
-  if (elems[0].getStyle("display") != "table-cell") {
-    var parentNode = elems[0].parentNode;
-    var table = document.createElement("table");
-    table.style.width = "100%";
-    var tbody = document.createElement("tbody");
-    table.appendChild(tbody);
-    var tr = document.createElement("tr");
-    tbody.appendChild(tr);
-    parentNode.appendChild(table);
-    for(var i=0; i<elems.length; i++) {
-      parentNode.removeChild(elems[i]);
-      var td = document.createElement("td");
-      td.appendChild(elems[i]);
-      tr.appendChild(td);
-    }
-  }
-}
-function fixbanner() {
-  // Use prototype api methods here
-  var elems = [$('room_banner'),$('actiondiv')];
-  fixMissingCSSTable(elems);
-  if (!is_ie6) {
-    Event.observe(window, 'resize', makeContentScrollable);
-    makeContentScrollable();
-  }
-}
-function makeContentScrollable() {
-if (document.getElementById("banner")
-      && document.getElementById("content") 
-      && !document.getElementById("summary_view")) {
-  WCLog("makeContentScrollable");
-    document.body.style.overflowY="hidden";
-    var global = $("global");
-    global.className += "scrollable";
-    var content = document.getElementById("content");
-    var banner = document.getElementById("banner");
-    var bannerHeight = banner.offsetHeight;
-    banner.style.width="100%";
-    var bannerPercent = (bannerHeight/document.viewport.getHeight())*100;
-    //banner.style.height=bannerPercent+"%";
-    content.style.overflowY="scroll";
-    //content.style.top=bannerPercent+"%";
-    content.style.height=(100-bannerPercent)+"%";
-    content.style.right="0px";
-  }
-}
-function fixOffsetBanner() {
-  var banner = document.getElementById("banner");
-  if (banner.offsetLeft > 0) {
-    var viewportWidth = document.viewport.getWidth();
-    var iconbarWidth = document.getElementById("iconbar").offsetWidth;
-    var contentDiv = document.getElementById("content");
-    var newContentWidth = viewportWidth-iconbarWidth;
-    contentDiv.style.width = newContentWidth+"px";
-  }
-}
-/** Attempt to stop overflowing in x-axis in IE6 */
-function resizeViewport() {
-  var documentWidth = 0;
-  var viewportWidth = document.viewport.getWidth();
-  var iconbar = $('iconbar');
-  var global = $('global');
-  if (iconbar == null || global == null || document.documentElement == null) {
-    return;
-  }
-  if (typeof window.offsetWidth != 'undefined') {
-    documentWidth = window.offsetWidth;
-  } else {
-    documentWidth = document.documentElement.offsetWidth;
-  }
-  if (documentWidth > viewportWidth) {
-    WCLog("resizeViewport");
-    document.documentElement.style.width = viewportWidth+"px";
-    document.documentElement.style.overflowX = "hidden";
-    //viewportWidth = 0.98 * viewportWidth;
-    var newIconBarSize = 0.16 * viewportWidth;
-    var newContentSize = viewportWidth - newIconBarSize;
-    iconbar.style.width = newIconBarSize+"px";
-    global.style.width = newContentSize+"px";
-  }
-}
-
 function RefreshSMTPqueueDisplay() {
 	new Ajax.Updater('smtpqueue_inner_div',
 	'display_smtpqueue_inner_div', { method: 'get',
