@@ -1,14 +1,19 @@
 /*
- * $Id: subst.c 8186 2010-01-01 15:44:15Z dothebart $
+ * $Id: subst.c 8593 2010-05-22 10:10:11Z dothebart $
  */
-
 #include "sysdep.h"
+
+
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <unistd.h>
 #include <dirent.h>
 #include <errno.h>
+
+#include <unistd.h>
+#include <stdio.h>
 #include <stdarg.h>
+#include <stddef.h>
+
 #define SHOW_ME_VAPPEND_PRINTF
 
 #include "webcit.h"
@@ -71,7 +76,7 @@ typedef struct _HashHandler {
 	WCHandlerFunc HandlerFunc;
 }HashHandler;
 
-void *load_template(StrBuf *filename, StrBuf *Key, HashList *PutThere);
+void *load_template(WCTemplate *NewTemplate);
 int EvaluateConditional(StrBuf *Target, int Neg, int state, WCTemplputParams *TP);
 
 
@@ -100,6 +105,7 @@ const char *CtxNames[]  = {
 	"Context MIME_ATACH",
 	"Context FILELIST",
 	"Context STRBUF",
+	"Context STRBUFARR",
 	"Context LONGVECTOR",
 	"Context ROOMS",
 	"Context FLOORS",
@@ -130,7 +136,6 @@ const char *ContextName(int ContextType)
 void LogTemplateError (StrBuf *Target, const char *Type, int ErrorPos, WCTemplputParams *TP, const char *Format, ...)
 {
 	wcsession *WCC;
-	StrBuf *Header;
 	StrBuf *Error;
 	StrBuf *Info;
         va_list arg_ptr;
@@ -186,7 +191,7 @@ void LogTemplateError (StrBuf *Target, const char *Type, int ErrorPos, WCTemplpu
 		return; 
 	}
 
-	Header = NewStrBuf();
+	if (WCC->WFBuf == NULL) WCC->WFBuf = NewStrBuf();
 	if (TP->Tokens != NULL) 
 	{
 		/* deprecated: 
@@ -207,7 +212,7 @@ void LogTemplateError (StrBuf *Target, const char *Type, int ErrorPos, WCTemplpu
 			     ChrPtr(TP->Tokens->FlatToken));
 
 
-		SerializeJson(Header, WildFireException(SKEY(TP->Tokens->FileName),
+		SerializeJson(WCC->WFBuf, WildFireException(SKEY(TP->Tokens->FileName),
 							TP->Tokens->Line,
 							Info,
 							1), 1);
@@ -217,7 +222,7 @@ void LogTemplateError (StrBuf *Target, const char *Type, int ErrorPos, WCTemplpu
 						      Error,
 						      eERROR), 1);
 */
-		WildFireSerializePayload(Header, WCC->HBuf, &WCC->Hdr->nWildfireHeaders, NULL);
+		
 	}
 	else
 	{
@@ -233,10 +238,8 @@ void LogTemplateError (StrBuf *Target, const char *Type, int ErrorPos, WCTemplpu
 			     Err, 
 			     ChrPtr(Error), 
 			     ChrPtr(TP->Tokens->FlatToken));
-		SerializeJson(Header, WildFireException(HKEY(__FILE__), __LINE__, Info, 1), 1);
-		WildFireSerializePayload(Header, WCC->HBuf, &WCC->Hdr->nWildfireHeaders, NULL);
+		SerializeJson(WCC->WFBuf, WildFireException(HKEY(__FILE__), __LINE__, Info, 1), 1);
 	}
-	FreeStrBuf(&Header);
 	FreeStrBuf(&Info);
 	FreeStrBuf(&Error);
 /*
@@ -251,7 +254,6 @@ void LogTemplateError (StrBuf *Target, const char *Type, int ErrorPos, WCTemplpu
 void LogError (StrBuf *Target, const char *Type, const char *Format, ...)
 {
 	wcsession *WCC;
-	StrBuf *Header;
 	StrBuf *Error;
 	StrBuf *Info;
         va_list arg_ptr;
@@ -266,16 +268,13 @@ void LogError (StrBuf *Target, const char *Type, const char *Format, ...)
 	lprintf(1, ChrPtr(Error));
 
 	WCC = WC;
-	Header = NewStrBuf();
+	if (WCC->WFBuf == NULL) WCC->WFBuf = NewStrBuf();
 
+	SerializeJson(WCC->WFBuf, WildFireException(Type, strlen(Type),
+						    0,
+						    Info,
+						    1), 1);
 
-	SerializeJson(Header, WildFireException(Type, strlen(Type),
-						0,
-						Info,
-						1), 1);
-	WildFireSerializePayload(Header, WCC->HBuf, &WCC->Hdr->nWildfireHeaders, NULL);
-	
-	FreeStrBuf(&Header);
 	FreeStrBuf(&Info);
 	FreeStrBuf(&Error);
 /*
@@ -449,7 +448,7 @@ void clear_local_substs(void) {
 	clear_substs (WC);
 }
 
-int NeedNewBuf(type)
+int NeedNewBuf(int type)
 {
 	switch(type) {
 	case WCS_STRING:
@@ -761,6 +760,31 @@ void pvo_do_cmd(StrBuf *Target, StrBuf *servcmd) {
 	}
 }
 
+int HaveTemplateTokenString(StrBuf *Target, 
+			    WCTemplputParams *TP,
+			    int N,
+			    const char **Value, 
+			    long *len)
+{
+	if (N >= TP->Tokens->nParameters) {
+		return 0;
+	}
+
+	switch (TP->Tokens->Params[N]->Type) {
+	case TYPE_INTDEFINE:
+	case TYPE_STR:
+	case TYPE_BSTR:
+	case TYPE_PREFSTR:
+	case TYPE_GETTEXT:
+	case TYPE_SUBTEMPLATE:
+		return 1;
+	case TYPE_LONG:
+	case TYPE_PREFINT:
+	default:
+		return 0;
+	}
+}
+
 void GetTemplateTokenString(StrBuf *Target, 
 			    WCTemplputParams *TP,
 			    int N,
@@ -1096,16 +1120,22 @@ void PutNewToken(WCTemplate *Template, WCTemplateToken *NewToken)
 		if (Template->TokenSpace <= 0) {
 			Template->Tokens = (WCTemplateToken**)malloc(
 				sizeof(WCTemplateToken*) * 10);
-			memset(Template->Tokens, 0, sizeof(WCTemplateToken*));
+			memset(Template->Tokens, 0, sizeof(WCTemplateToken*) * 10);
 			Template->TokenSpace = 10;
 		}
 		else {
 			WCTemplateToken **NewTokens;
-			NewTokens= (WCTemplateToken**)malloc(
-				sizeof(WCTemplateToken*) * 
-				Template->TokenSpace * 2);
-			memcpy(NewTokens, Template->Tokens, 
+
+			NewTokens= (WCTemplateToken**) malloc(
+				sizeof(WCTemplateToken*) * Template->TokenSpace * 2);
+
+			memset(NewTokens, 
+			       0, sizeof(WCTemplateToken*) * Template->TokenSpace * 2);
+
+			memcpy(NewTokens, 
+			       Template->Tokens, 
 			       sizeof(WCTemplateToken*) * Template->nTokensUsed);
+
 			free(Template->Tokens);
 			Template->TokenSpace *= 2;
 			Template->Tokens = NewTokens;
@@ -1258,13 +1288,16 @@ TemplateParam *GetNextParameter(StrBuf *Buf,
 	       (*pch == ',' )||
 	       (*pch == '\n')) pch ++;
 
-	if (DumpTemplateI18NStrings && (Parm->Type == TYPE_GETTEXT)) {
-		StrBufAppendPrintf(I18nDump, "_(\"%s\");\n", Parm->Start);
-	}
-	if (Parm->Type == TYPE_INTDEFINE)
+	switch (Parm->Type)
 	{
+	case TYPE_GETTEXT:
+		if (DumpTemplateI18NStrings) {
+			StrBufAppendPrintf(I18nDump, "_(\"%s\");\n", Parm->Start);
+		}
+		break;
+	case TYPE_INTDEFINE: {
 		void *vPVal;
-
+		
 		if (GetHash(Defines, Parm->Start, Parm->len, &vPVal) &&
 		    (vPVal != NULL))
 		{
@@ -1278,7 +1311,18 @@ TemplateParam *GetNextParameter(StrBuf *Buf,
 			LogTemplateError(NULL, "Define", ERR_PARM1, TP,
 					 "%s isn't known!!",
 					 Parm->Start);
-		}
+		}}
+		break;
+	case TYPE_SUBTEMPLATE:{
+		void *vTmpl;
+		/* well, we don't check the mobile stuff here... */
+		if (!GetHash(LocalTemplateCache, Parm->Start, Parm->len, &vTmpl) &&
+		    !GetHash(TemplateCache, Parm->Start, Parm->len, &vTmpl)) {
+			LogTemplateError(
+				NULL, "SubTemplate", ERR_PARM1, TP,
+				"referenced here doesn't exist");
+		}}
+		break;
 	}
 	*pCh = pch;
 	return Parm;
@@ -1286,8 +1330,8 @@ TemplateParam *GetNextParameter(StrBuf *Buf,
 
 WCTemplateToken *NewTemplateSubstitute(StrBuf *Buf, 
 				       const char *pStart, 
-				       const char *pTmplStart, 
-				       const char *pTmplEnd, 
+				       const char *pTokenStart, 
+				       const char *pTokenEnd, 
 				       long Line,
 				       WCTemplate *pTmpl)
 {
@@ -1303,34 +1347,34 @@ WCTemplateToken *NewTemplateSubstitute(StrBuf *Buf,
 	NewToken->FileName = pTmpl->FileName; /* to print meaningfull log messages... */
 	NewToken->Flags = 0;
 	NewToken->Line = Line + 1;
-	NewToken->pTokenStart = pTmplStart;
-	NewToken->TokenStart = pTmplStart - pStart;
-	NewToken->TokenEnd =  (pTmplEnd - pStart) - NewToken->TokenStart;
-	NewToken->pTokenEnd = pTmplEnd;
+	NewToken->pTokenStart = pTokenStart;
+	NewToken->TokenStart = pTokenStart - pStart;
+	NewToken->TokenEnd =  (pTokenEnd - pStart) - NewToken->TokenStart;
+	NewToken->pTokenEnd = pTokenEnd;
 	NewToken->NameEnd = NewToken->TokenEnd - 2;
 	NewToken->PreEval = NULL;
-	NewToken->FlatToken = NewStrBufPlain(pTmplStart + 2, pTmplEnd - pTmplStart - 2);
+	NewToken->FlatToken = NewStrBufPlain(pTokenStart + 2, pTokenEnd - pTokenStart - 2);
 	StrBufShrinkToFit(NewToken->FlatToken, 1);
 
-	StrBufPeek(Buf, pTmplStart, + 1, '\0');
-	StrBufPeek(Buf, pTmplEnd, -1, '\0');
-	pch = NewToken->pName = pTmplStart + 2;
+	StrBufPeek(Buf, pTokenStart, + 1, '\0');
+	StrBufPeek(Buf, pTokenEnd, -1, '\0');
+	pch = NewToken->pName = pTokenStart + 2;
 
 	NewToken->HaveParameters = 0;;
 	NewToken->nParameters = 0;
 
-	while (pch < pTmplEnd - 1) {
+	while (pch < pTokenEnd - 1) {
 		if (*pch == '(') {
 			StrBufPeek(Buf, pch, -1, '\0');
 			NewToken->NameEnd = pch - NewToken->pName;
 			pch ++;
-			if (*(pTmplEnd - 1) != ')') {
+			if (*(pTokenEnd - 1) != ')') {
 				LogTemplateError(
 					NULL, "Parseerror", ERR_NAME, &TP, 
 					"Warning, Non welformed Token; missing right parenthesis");
 			}
-			while (pch < pTmplEnd - 1) {
-				Param = GetNextParameter(Buf, &pch, pTmplEnd - 1, NewToken, pTmpl, &TP);
+			while (pch < pTokenEnd - 1) {
+				Param = GetNextParameter(Buf, &pch, pTokenEnd - 1, NewToken, pTmpl, &TP);
 				if (Param != NULL) {
 					NewToken->HaveParameters = 1;
 					if (NewToken->nParameters > MAXPARAM) {
@@ -1409,13 +1453,29 @@ WCTemplateToken *NewTemplateSubstitute(StrBuf *Buf,
 				NewToken->nParameters);
 			break;
 		}
+		else {
+			void *vTmpl;
+			/* well, we don't check the mobile stuff here... */
+			if (!GetHash(LocalTemplateCache, 
+				     NewToken->Params[0]->Start, 
+				     NewToken->Params[0]->len, 
+				     &vTmpl) &&
+			    !GetHash(TemplateCache, 
+				     NewToken->Params[0]->Start, 
+				     NewToken->Params[0]->len, 
+				     &vTmpl)) {
+				LogTemplateError(
+					NULL, "SubTemplate", ERR_PARM1, &TP,
+					"doesn't exist");
+			}
+		}
 		break;
 	case SV_CUST_STR_CONDITIONAL:
 	case SV_CONDITIONAL:
 	case SV_NEG_CONDITIONAL:
 		if (NewToken->nParameters <2) {
 			LogTemplateError(
-				NULL, "Conditional", ERR_NAME, &TP,
+				NULL, "Conditional", ERR_PARM1, &TP,
 				"require at least 2 parameters, you gave %d params", 
 				NewToken->nParameters);
 			NewToken->Flags = 0;
@@ -1423,7 +1483,7 @@ WCTemplateToken *NewTemplateSubstitute(StrBuf *Buf,
 		}
 		if (NewToken->Params[1]->lvalue == 0) {
 			LogTemplateError(
-				NULL, "Conditional", ERR_NAME, &TP,
+				NULL, "Conditional", ERR_PARM1, &TP,
 				"Conditional ID (Parameter 1) mustn't be 0!");
 			NewToken->Flags = 0;
 			break;
@@ -1437,7 +1497,7 @@ WCTemplateToken *NewTemplateSubstitute(StrBuf *Buf,
 			    (NewToken->Params[0]->Start[0] == 'X'))
 				break;
 			LogTemplateError(
-				NULL, "Conditional", ERR_NAME, &TP,
+				NULL, "Conditional", ERR_PARM1, &TP,
 				"Not found!");
 /*
 			NewToken->Error = NewStrBuf();
@@ -1483,6 +1543,10 @@ void *prepare_template(StrBuf *filename, StrBuf *Key, HashList *PutThere)
 		StrBufAppendBufPlain(NewTemplate->MimeType, HKEY("; charset=utf-8"), 0);
 	}
 
+	if (strstr(ChrPtr(NewTemplate->MimeType), "text") != NULL) {
+		StrBufAppendBufPlain(NewTemplate->MimeType, HKEY("; charset=utf-8"), 0);
+	}
+
 	Put(PutThere, ChrPtr(Key), StrLength(Key), NewTemplate, FreeWCTemplate);
 	return NewTemplate;
 }
@@ -1491,45 +1555,53 @@ void *prepare_template(StrBuf *filename, StrBuf *Key, HashList *PutThere)
  * \brief Display a variable-substituted template
  * \param templatename template file to load
  */
-void *load_template(StrBuf *filename, StrBuf *Key, HashList *PutThere)
+void *duplicate_template(WCTemplate *OldTemplate)
+{
+	WCTemplate *NewTemplate;
+
+	NewTemplate = (WCTemplate *) malloc(sizeof(WCTemplate));
+	memset(NewTemplate, 0, sizeof(WCTemplate));
+	NewTemplate->Data = NULL;
+	NewTemplate->FileName = NewStrBufDup(OldTemplate->FileName);
+	StrBufShrinkToFit(NewTemplate->FileName, 1);
+	NewTemplate->nTokensUsed = 0;
+	NewTemplate->TokenSpace = 0;
+	NewTemplate->Tokens = NULL;
+	NewTemplate->MimeType = NewStrBufDup(OldTemplate->MimeType);
+	return NewTemplate;
+}
+
+/**
+ * \brief Display a variable-substituted template
+ * \param templatename template file to load
+ */
+void *load_template(WCTemplate *NewTemplate)
 {
 	int fd;
 	struct stat statbuf;
 	const char *pS, *pE, *pch, *Err;
 	long Line;
 	int pos;
-	WCTemplate *NewTemplate;
 
-	fd = open(ChrPtr(filename), O_RDONLY);
+	fd = open(ChrPtr(NewTemplate->FileName), O_RDONLY);
 	if (fd <= 0) {
 		lprintf(1, "ERROR: could not open template '%s' - %s\n",
-			ChrPtr(filename), strerror(errno));
+			ChrPtr(NewTemplate->FileName), strerror(errno));
 		return NULL;
 	}
 
 	if (fstat(fd, &statbuf) == -1) {
 		lprintf(1, "ERROR: could not stat template '%s' - %s\n",
-			ChrPtr(filename), strerror(errno));
+			ChrPtr(NewTemplate->FileName), strerror(errno));
 		return NULL;
 	}
 
-	NewTemplate = (WCTemplate *) malloc(sizeof(WCTemplate));
-	memset(NewTemplate, 0, sizeof(WCTemplate));
-	NewTemplate->Data = NewStrBufPlain(NULL, statbuf.st_size);
-	NewTemplate->FileName = NewStrBufDup(filename);
-	NewTemplate->nTokensUsed = 0;
-	NewTemplate->TokenSpace = 0;
-	NewTemplate->Tokens = NULL;
-	NewTemplate->MimeType = NewStrBufPlain(GuessMimeByFilename (SKEY(NewTemplate->FileName)), -1);
-	if (strstr(ChrPtr(NewTemplate->MimeType), "text") != NULL) {
-		StrBufAppendBufPlain(NewTemplate->MimeType, HKEY("; charset=utf-8"), 0);
-	}
-
+	NewTemplate->Data = NewStrBufPlain(NULL, statbuf.st_size + 1);
 	if (StrBufReadBLOB(NewTemplate->Data, &fd, 1, statbuf.st_size, &Err) < 0) {
 		close(fd);
-		FreeWCTemplate(NewTemplate);
 		lprintf(1, "ERROR: reading template '%s' - %s<br />\n",
-			ChrPtr(filename), strerror(errno));
+			ChrPtr(NewTemplate->FileName), strerror(errno));
+		//FreeWCTemplate(NewTemplate);/////tODO
 		return NULL;
 	}
 	close(fd);
@@ -1578,8 +1650,6 @@ void *load_template(StrBuf *filename, StrBuf *Key, HashList *PutThere)
 			    NewTemplateSubstitute(NewTemplate->Data, pS, pts, pte, Line, NewTemplate));
 		pch ++;
 	}
-	if (LoadTemplates == 0)
-		Put(PutThere, ChrPtr(Key), StrLength(Key), NewTemplate, FreeWCTemplate);
 	return NewTemplate;
 }
 
@@ -1592,74 +1662,212 @@ const char* PrintTemplate(void *vSubst)
 
 }
 
-int LoadTemplateDir(const char *DirName, HashList *wireless, HashList *big)
+int LoadTemplateDir(const StrBuf *DirName, HashList *wireless, HashList *big, const StrBuf *BaseKey)
 {
+	int Toplevel;
 	StrBuf *FileName;
-	StrBuf *Tag;
-	StrBuf *Dir;
+	StrBuf *Key;
+	StrBuf *SubKey;
+	StrBuf *SubDirectory;
 	DIR *filedir = NULL;
 	struct dirent *filedir_entry;
+	struct dirent *d;
+	int d_type = 0;
 	int d_namelen;
 	int d_without_ext;
 	int IsMobile;
 	
-	Dir = NewStrBuf();
-	StrBufPrintf(Dir, "%s/t", DirName);
-	filedir = opendir (ChrPtr(Dir));
-	if (filedir == NULL) {
-		FreeStrBuf(&Dir);
+	d = (struct dirent *)malloc(offsetof(struct dirent, d_name) + PATH_MAX + 1);
+	if (d == NULL) {
 		return 0;
 	}
 
-	FileName = NewStrBuf();
-	Tag = NewStrBuf();
-	while ((filedir_entry = readdir(filedir)))
+	filedir = opendir (ChrPtr(DirName));
+	if (filedir == NULL) {
+		free(d);
+		return 0;
+	}
+
+	Toplevel = StrLength(BaseKey) == 0;
+	SubDirectory = NewStrBuf();
+	SubKey = NewStrBuf();
+	FileName = NewStrBufPlain(NULL, PATH_MAX);
+	Key = NewStrBuf();
+	while ((readdir_r(filedir, d, &filedir_entry) == 0) &&
+	       (filedir_entry != NULL))
 	{
 		char *MinorPtr;
 		char *PStart;
 #ifdef _DIRENT_HAVE_D_NAMELEN
 		d_namelen = filedir_entry->d_namelen;
+		d_type = filedir_entry->d_type;
 #else
+
+#ifndef DT_UNKNOWN
+#define DT_UNKNOWN     0
+#define DT_DIR         4
+#define DT_REG         8
+#define DT_LNK         10
+
+#define IFTODT(mode)   (((mode) & 0170000) >> 12)
+#define DTTOIF(dirtype)        ((dirtype) << 12)
+#endif
 		d_namelen = strlen(filedir_entry->d_name);
+		d_type = DT_UNKNOWN;
 #endif
 		d_without_ext = d_namelen;
-		while ((d_without_ext > 0) && (filedir_entry->d_name[d_without_ext] != '.'))
-			d_without_ext --;
-		if ((d_without_ext == 0) || (d_namelen < 3))
-			continue;
+
 		if ((d_namelen > 1) && filedir_entry->d_name[d_namelen - 1] == '~')
 			continue; /* Ignore backup files... */
 
-		IsMobile = (strstr(filedir_entry->d_name, ".m.html")!= NULL);
-		PStart = filedir_entry->d_name;
-		StrBufPrintf(FileName, "%s/%s", ChrPtr(Dir),  filedir_entry->d_name);
-		MinorPtr = strchr(filedir_entry->d_name, '.');
-		if (MinorPtr != NULL)
-			*MinorPtr = '\0';
-		StrBufPlain(Tag, filedir_entry->d_name, MinorPtr - filedir_entry->d_name);
+		if ((d_namelen == 1) && 
+		    (filedir_entry->d_name[0] == '.'))
+			continue;
 
-		if (LoadTemplates > 1)
-			lprintf(1, "%s %d %s\n",ChrPtr(FileName), IsMobile, ChrPtr(Tag));
-		if (LoadTemplates == 0)
-			load_template(FileName, Tag, (IsMobile)?wireless:big);
-		else
-			prepare_template(FileName, Tag, (IsMobile)?wireless:big);
+		if ((d_namelen == 2) && 
+		    (filedir_entry->d_name[0] == '.') &&
+		    (filedir_entry->d_name[1] == '.'))
+			continue;
+
+		if (d_type == DT_UNKNOWN) {
+			struct stat s;
+			char path[PATH_MAX];
+			snprintf(path, PATH_MAX, "%s/%s", 
+				 ChrPtr(DirName), filedir_entry->d_name);
+			if (stat(path, &s) == 0) {
+				d_type = IFTODT(s.st_mode);
+			}
+		}
+		switch (d_type)
+		{
+		case DT_DIR:
+			/* Skip directories we are not interested in... */
+			if (strcmp(filedir_entry->d_name, ".svn") == 0)
+				continue;
+
+			FlushStrBuf(SubKey);
+			if (!Toplevel) {
+				/* If we're not toplevel, the upper dirs count as foo_bar_<local name>*/
+				StrBufAppendBuf(SubKey, BaseKey, 0);
+				StrBufAppendBufPlain(SubKey, HKEY("_"), 0);
+			}
+			StrBufAppendBufPlain(SubKey, filedir_entry->d_name, d_namelen, 0);
+
+			FlushStrBuf(SubDirectory);
+			StrBufAppendBuf(SubDirectory, DirName, 0);
+			if (ChrPtr(SubDirectory)[StrLength(SubDirectory) - 1] != '/')
+				StrBufAppendBufPlain(SubDirectory, HKEY("/"), 0);
+			StrBufAppendBufPlain(SubDirectory, filedir_entry->d_name, d_namelen, 0);
+
+			LoadTemplateDir(SubDirectory, wireless, big, SubKey);
+
+			break;
+		case DT_LNK: /* TODO: check whether its a file or a directory */
+		case DT_REG:
+
+
+			while ((d_without_ext > 0) && (filedir_entry->d_name[d_without_ext] != '.'))
+				d_without_ext --;
+			if ((d_without_ext == 0) || (d_namelen < 3))
+				continue;
+			if (((d_namelen > 1) && filedir_entry->d_name[d_namelen - 1] == '~') ||
+			    (strcmp(&filedir_entry->d_name[d_without_ext], ".orig") == 0) ||
+			    (strcmp(&filedir_entry->d_name[d_without_ext], ".swp") == 0))
+				continue; /* Ignore backup files... */
+			/* .m.xxx is for mobile useragents! */
+			IsMobile = 0;
+			if (d_without_ext > 2)
+				IsMobile = (filedir_entry->d_name[d_without_ext - 1] == 'm') &&
+					(filedir_entry->d_name[d_without_ext - 2] == '.');
+			PStart = filedir_entry->d_name;
+			StrBufPrintf(FileName, "%s/%s", ChrPtr(DirName),  filedir_entry->d_name);
+			MinorPtr = strchr(filedir_entry->d_name, '.');
+			if (MinorPtr != NULL)
+				*MinorPtr = '\0';
+			FlushStrBuf(Key);
+			if (!Toplevel) {
+				/* If we're not toplevel, the upper dirs count as foo_bar_<local name>*/
+				StrBufAppendBuf(Key, BaseKey, 0);
+				StrBufAppendBufPlain(Key, HKEY("_"), 0);
+			}
+			StrBufAppendBufPlain(Key, filedir_entry->d_name, MinorPtr - filedir_entry->d_name, 0);
+
+			if (LoadTemplates >= 1)
+				lprintf(1, "%s %d %s\n", ChrPtr(FileName), IsMobile, ChrPtr(Key));
+			prepare_template(FileName, Key, (IsMobile)?wireless:big);
+		default:
+			break;
+		}
 	}
+	free(d);
 	closedir(filedir);
 	FreeStrBuf(&FileName);
-	FreeStrBuf(&Tag);
-	FreeStrBuf(&Dir);
+	FreeStrBuf(&Key);
+	FreeStrBuf(&SubDirectory);
+	FreeStrBuf(&SubKey);
 	return 1;
 }
 
 void InitTemplateCache(void)
 {
-	LoadTemplateDir(static_dirs[0],
+	int i;
+	StrBuf *Key;
+	StrBuf *Dir;
+	HashList *Templates[4];
+
+	Dir = NewStrBuf();
+	Key = NewStrBuf();
+
+	/* Primary Template set... */
+	StrBufPrintf(Dir, "%s/t", static_dirs[0]);
+	LoadTemplateDir(Dir,
 			WirelessTemplateCache,
-			TemplateCache);
-	LoadTemplateDir(static_dirs[1],
+			TemplateCache, 
+			Key);
+
+	/* User local Template set */
+	StrBufPrintf(Dir, "%s/t", static_dirs[1]);
+	LoadTemplateDir(Dir,
 			WirelessLocalTemplateCache,
-			LocalTemplateCache);
+			LocalTemplateCache, 
+			Key);
+	
+	/* Debug Templates, just to be loaded while debugging. */
+	
+	StrBufPrintf(Dir, "%s/dbg", static_dirs[0]);
+	LoadTemplateDir(Dir,
+			WirelessTemplateCache,
+			TemplateCache, 
+			Key);
+	Templates[0] = WirelessTemplateCache;
+	Templates[1] = TemplateCache;
+	Templates[2] = WirelessLocalTemplateCache;
+	Templates[3] = LocalTemplateCache;
+
+
+	if (LoadTemplates == 0) 
+		for (i=0; i < 4; i++) {
+			const char *Key;
+			long KLen;
+			HashPos *At;
+			void *vTemplate;
+
+			At = GetNewHashPos(Templates[i], 0);
+			while (GetNextHashPos(Templates[i], 
+					      At, 
+					      &KLen,
+					      &Key, 
+					      &vTemplate) && 
+			       (vTemplate != NULL))
+			{
+				load_template((WCTemplate *)vTemplate);
+			}
+			DeleteHashPos(&At);
+		}
+
+
+	FreeStrBuf(&Dir);
+	FreeStrBuf(&Key);
 }
 
 
@@ -1768,13 +1976,13 @@ const StrBuf *ProcessTemplate(WCTemplate *Tmpl, StrBuf *Target, WCTemplputParams
 		if (LoadTemplates > 1)
 			lprintf(1, "DBG: ----- loading:  [%s] ------ \n", 
 				ChrPtr(Tmpl->FileName));
-
-		pTmpl = load_template(Tmpl->FileName, NULL, NULL);
-		if(pTmpl == NULL) {
+		pTmpl = duplicate_template(Tmpl);
+		if(load_template(pTmpl) == NULL) {
 			StrBufAppendPrintf(
 				Target, 
 				"<pre>\nError loading Template [%s]\n See Logfile for details\n</pre>\n", 
 				ChrPtr(Tmpl->FileName));
+			FreeWCTemplate(pTmpl);
 			return NULL;
 		}
 
@@ -1931,6 +2139,7 @@ int preeval_iterate(WCTemplateToken *Token)
 {
 	WCTemplputParams TPP;
 	WCTemplputParams *TP;
+	void *vTmpl;
 	void *vIt;
 
 	memset(&TPP, 0, sizeof(WCTemplputParams));
@@ -1938,10 +2147,22 @@ int preeval_iterate(WCTemplateToken *Token)
 	TP->Tokens = Token;
 	if (!GetHash(Iterators, TKEY(0), &vIt)) {
 		LogTemplateError(
-			NULL, "Iterator", ERR_NAME, TP,
+			NULL, "Iterator", ERR_PARM1, TP,
 			"not found");
 		return 0;
 	}
+	if (TP->Tokens->Params[1]->Type != TYPE_SUBTEMPLATE) {
+		LogTemplateError(NULL, "Iterator", ERR_PARM1, TP,
+				 "Need token with type Subtemplate as param 1, have %s", 
+				 TP->Tokens->Params[1]->Start);
+	}
+	
+	/* well, we don't check the mobile stuff here... */
+	if (!GetHash(LocalTemplateCache, TKEY(1), &vTmpl) &&
+	    !GetHash(TemplateCache, TKEY(1), &vTmpl)) {
+		LogTemplateError(NULL, "SubTemplate", ERR_PARM1, TP,
+				 "referenced here doesn't exist");
+	}
 	Token->Preeval2 = vIt;
 	return 1;
 }
@@ -2215,6 +2436,39 @@ void RegisterTokenParamDefine(const char *Name, long len,
 	Put(Defines, Name, len, PVal, NULL);
 }
 
+long GetTokenDefine(const char *Name, long len, 
+		    long DefValue)
+{
+	void *vPVal;
+
+	if (GetHash(Defines, Name, len, &vPVal) &&
+	     (vPVal != NULL))
+	 {
+		 return *(long*) vPVal;
+	 }
+	 else
+	 {
+		 return DefValue;
+	 }
+}
+
+void tmplput_DefStr(StrBuf *Target, WCTemplputParams *TP)
+{
+	const char *Str;
+	long len;
+	GetTemplateTokenString(Target, TP, 2, &Str, &len);
+	
+	StrBufAppendBufPlain(Target, Str, len, 0);
+}
+
+void tmplput_DefVal(StrBuf *Target, WCTemplputParams *TP)
+{
+	int val;
+
+	val = GetTemplateTokenNumber(Target, TP, 0, 0);
+	StrBufAppendPrintf(Target, "%d", val);
+}
+
 HashList *Defines;
 
 /*-----------------------------------------------------------------------------
@@ -2234,6 +2488,36 @@ int ConditionalContextStr(StrBuf *Target, WCTemplputParams *TP)
 	return strcmp(ChrPtr(TokenText), CompareToken) == 0;
 }
 
+void tmplput_ContextStringArray(StrBuf *Target, WCTemplputParams *TP)
+{
+	HashList *Arr = (HashList*) CTX;
+	void *pV;
+	int val;
+
+	val = GetTemplateTokenNumber(Target, TP, 0, 0);
+	if (GetHash(Arr, IKEY(val), &pV) && 
+	    (pV != NULL)) {
+		StrBufAppendTemplate(Target, TP, (StrBuf*)pV, 1);
+	}
+}
+int ConditionalContextStrinArray(StrBuf *Target, WCTemplputParams *TP)
+{
+	HashList *Arr = (HashList*) CTX;
+	void *pV;
+	int val;
+	const char *CompareToken;
+	long len;
+
+	GetTemplateTokenString(Target, TP, 2, &CompareToken, &len);
+	val = GetTemplateTokenNumber(Target, TP, 0, 0);
+	if (GetHash(Arr, IKEY(val), &pV) && 
+	    (pV != NULL)) {
+		return strcmp(ChrPtr((StrBuf*)pV), CompareToken) == 0;
+	}
+	else
+		return 0;
+}
+
 /*-----------------------------------------------------------------------------
  *                      Boxed-API
  */
@@ -2272,6 +2556,68 @@ void tmpl_do_boxed(StrBuf *Target, WCTemplputParams *TP)
 /*-----------------------------------------------------------------------------
  *                      Tabbed-API
  */
+int preeval_do_tabbed(WCTemplateToken *Token)
+{
+	WCTemplputParams TPP;
+	WCTemplputParams *TP;
+	const char *Ch;
+	long len;
+	int i, nTabs;
+
+
+	memset(&TPP, 0, sizeof(WCTemplputParams));
+	TP = &TPP;
+	TP->Tokens = Token;
+	nTabs = TP->Tokens->nParameters / 2 - 1;
+	if (TP->Tokens->nParameters % 2 != 0)
+	{
+		LogTemplateError(NULL, "TabbedApi", ERR_PARM1, TP,
+				 "need even number of arguments");
+		return 0;
+
+	}
+	else for (i = 0; i < nTabs; i++) {
+		if (!HaveTemplateTokenString(NULL, 
+					     TP, 
+					     i * 2,
+					     &Ch,
+					     &len) || 
+		    (TP->Tokens->Params[i * 2]->len == 0))
+		{
+			LogTemplateError(NULL, "TabbedApi", ERR_PARM1, TP,
+					 "Tab-Subject %d needs to be able to produce a string, have %s", 
+					 i, TP->Tokens->Params[i * 2]->Start);
+			return 0;
+		}
+		if (!HaveTemplateTokenString(NULL, 
+					     TP, 
+					     i * 2 + 1,
+					     &Ch,
+					     &len) || 
+		    (TP->Tokens->Params[i * 2 + 1]->len == 0))
+		{
+			LogTemplateError(NULL, "TabbedApi", ERR_PARM1, TP,
+					 "Tab-Content %d needs to be able to produce a string, have %s", 
+					 i, TP->Tokens->Params[i * 2 + 1]->Start);
+			return 0;
+		}
+	}
+
+	if (!HaveTemplateTokenString(NULL, 
+				     TP, 
+				     i * 2 + 1,
+				     &Ch,
+				     &len) || 
+	    (TP->Tokens->Params[i * 2 + 1]->len == 0))
+	{
+		LogTemplateError(NULL, "TabbedApi", ERR_PARM1, TP,
+				 "Tab-Content %d needs to be able to produce a string, have %s", 
+				 i, TP->Tokens->Params[i * 2 + 1]->Start);
+		return 0;
+	}
+	return 1;
+}
+
 
 void tmpl_do_tabbed(StrBuf *Target, WCTemplputParams *TP)
 {
@@ -2310,6 +2656,8 @@ void tmpl_do_tabbed(StrBuf *Target, WCTemplputParams *TP)
 		DoTemplate(TKEY(i * 2 + 1), Target, TP);
 		StrEndTab(Target, i, nTabs);
 	}
+	for (i = 0; i < ntabs; i++) 
+		FreeStrBuf(&TabNames[i]);
 }
 
 
@@ -2686,12 +3034,16 @@ InitModule_SUBST
 	RegisterNamespace("SORT:ORDER", 1, 2, tmplput_SORT_ORDER, NULL, CTX_NONE);
 	RegisterNamespace("SORT:NEXT", 1, 2, tmplput_SORT_NEXT, NULL, CTX_NONE);
 	RegisterNamespace("CONTEXTSTR", 0, 1, tmplput_ContextString, NULL, CTX_STRBUF);
+	RegisterNamespace("CONTEXTSTRARR", 1, 2, tmplput_ContextStringArray, NULL, CTX_STRBUF);
 	RegisterNamespace("ITERATE", 2, 100, tmpl_iterate_subtmpl, preeval_iterate, CTX_NONE);
 	RegisterNamespace("DOBOXED", 1, 2, tmpl_do_boxed, NULL, CTX_NONE);
-	RegisterNamespace("DOTABBED", 2, 100, tmpl_do_tabbed, NULL, CTX_NONE);
+	RegisterNamespace("DOTABBED", 2, 100, tmpl_do_tabbed, preeval_do_tabbed, CTX_NONE);
 	RegisterNamespace("LONGVECTOR", 1, 1, tmplput_long_vector, NULL, CTX_LONGVECTOR);
+
+
 	RegisterConditional(HKEY("COND:SUBST"), 3, ConditionalVar, CTX_NONE);
 	RegisterConditional(HKEY("COND:CONTEXTSTR"), 3, ConditionalContextStr, CTX_STRBUF);
+	RegisterConditional(HKEY("COND:CONTEXTSTRARR"), 4, ConditionalContextStrinArray, CTX_STRBUFARR);
 	RegisterConditional(HKEY("COND:LONGVECTOR"), 4, ConditionalLongVector, CTX_LONGVECTOR);
 
 	RegisterControlConditional(HKEY("COND:ITERATE:ISGROUPCHANGE"), 2, 
@@ -2708,6 +3060,10 @@ InitModule_SUBST
 	RegisterControlNS(HKEY("ITERATE:KEY"), 0, 0, tmplput_ITERATE_KEY, CTX_ITERATE);
 	RegisterControlNS(HKEY("ITERATE:N"), 0, 0, tmplput_ITERATE_LASTN, CTX_ITERATE);
 	RegisterNamespace("CURRENTFILE", 0, 1, tmplput_CURRENT_FILE, NULL, CTX_NONE);
+	RegisterNamespace("DEF:STR", 1, 1, tmplput_DefStr, NULL, CTX_NONE);
+	RegisterNamespace("DEF:VAL", 1, 1, tmplput_DefVal, NULL, CTX_NONE);
+
+
 
 
 }
@@ -2771,6 +3127,7 @@ SessionDetachModule_SUBST
 (wcsession *sess)
 {
 	DeleteHash(&sess->vars);
+	FreeStrBuf(&sess->WFBuf);
 }
 
 void 
