@@ -1,5 +1,5 @@
 /*
- * $Id: summary.c 7939 2009-10-23 12:51:53Z dothebart $
+ * $Id: summary.c 8636 2010-06-22 16:23:11Z ajc $
  *
  * Displays the "Summary Page"
  */
@@ -95,11 +95,12 @@ void tasks_section(void) {
 	Buf = NewStrBufPlain(HKEY("_TASKS_"));
 	gotoroom(Buf);
 	FreeStrBuf(&Buf);
-	if (WCC->wc_view != VIEW_TASKS) {
+
+	if (WCC->CurRoom.view != VIEW_TASKS) {
 		num_msgs = 0;
 	}
 	else {
-		num_msgs = load_msg_ptrs("MSGS ALL", &Stat);
+		num_msgs = load_msg_ptrs("MSGS ALL", &Stat, NULL);
 	}
 
 	if (num_msgs > 0) {
@@ -143,11 +144,11 @@ void calendar_section(void) {
 	Buf = NewStrBufPlain(HKEY("_CALENDAR_"));
 	gotoroom(Buf);
 	FreeStrBuf(&Buf);
-	if ( (WC->wc_view != VIEW_CALENDAR) && (WC->wc_view != VIEW_CALBRIEF) ) {
+	if ( (WC->CurRoom.view != VIEW_CALENDAR) && (WC->CurRoom.view != VIEW_CALBRIEF) ) {
 		num_msgs = 0;
 	}
 	else {
-		num_msgs = load_msg_ptrs("MSGS ALL", &Stat);
+		num_msgs = load_msg_ptrs("MSGS ALL", &Stat, NULL);
 	}
 	calendar_GetParamsGetServerCall(&Stat, 
 					&v,
@@ -197,9 +198,9 @@ void server_info_section(void) {
  * not people I consider worthwhile, I still want them to use WebCit.
  */
 void summary_inner_div(void) {
-	wc_printf("<div class=\"fix_scrollbar_bug\">"
-		"<table width=\"100%%\" cellspacing=\"10\" cellpadding=\"0\">"
-		"<tr valign=top>");
+
+	wc_printf("<table width=\"98%%\" cellspacing=\"3\" cellpadding=\"0\">");
+	wc_printf("<tr valign=top>");
 
 	/*
 	 * Column One
@@ -251,7 +252,7 @@ void summary_inner_div(void) {
 	wc_printf(_("Who's&nbsp;online&nbsp;now"));
 	wc_printf("</div><div class=\"boxcontent\">");	
 	wc_printf("<div id=\"who_inner\">");	
-	do_template("wholistsummarysection", NULL);
+	do_template("who_summary", NULL);
 	wc_printf("</div></div></div>");
 	wc_printf("</td>");
 
@@ -283,22 +284,22 @@ void summary(void) {
 	char title[256];
 
 	output_headers(1, 1, 2, 0, 0, 0);
-	wc_printf("<div id=\"banner\">\n");
-	wc_printf("<div class=\"room_banner\">");
-        wc_printf("<img src=\"static/summscreen_48x.gif\">");
-        wc_printf("<h1>");
+	wc_printf("<div id=\"banner\" class=\"banner\">\n");
+	wc_printf("<table border=0><tr>");
+        wc_printf("<td><img src=\"static/summscreen_48x.gif\"></td>");
+        wc_printf("<td><h1>");
         snprintf(title, sizeof title, _("Summary page for %s"), ChrPtr(WC->wc_fullname));
         escputs(title);
         wc_printf("</h1><h2>");
         output_date();
-        wc_printf("</h2></div>");
+        wc_printf("</h2></td></tr></table>");
 	wc_printf("<div id=\"actiondiv\">");
 	wc_printf("<ul class=\"room_actions\">\n");
 	wc_printf("<li class=\"start_page\">");
 	offer_start_page(NULL, &NoCtx);
         wc_printf("</li></ul>");
-        wc_printf("</div>");
-        wc_printf("</div>");
+        wc_printf("</div>");	/* actiondiv */
+        wc_printf("</div>");	/* banner */
 
 	/*
 	 * You guessed it ... we're going to refresh using ajax.
@@ -317,7 +318,7 @@ void summary(void) {
 		"                            { method: 'get', frequency: 120 }  );	"
 		" new Ajax.PeriodicalUpdater('calendar_inner', 'calendar_inner_html',		"
 		"                            { method: 'get', frequency: 90 }  );	"
-		" new Ajax.PeriodicalUpdater('do_template', 'template=wholistsummarysection',	"
+		" new Ajax.PeriodicalUpdater('do_template', 'template=who_summary',	"
 		"                            { method: 'get', frequency: 30 }  );	"
 		"</script>							 	\n"
 	);
