@@ -1,5 +1,5 @@
 /*
- * $Id: useredit.c 8290 2010-02-09 20:01:49Z dothebart $
+ * $Id: useredit.c 8627 2010-06-17 20:38:03Z dothebart $
  */
 
 #include "webcit.h"
@@ -14,7 +14,7 @@
 void select_user_to_edit(const char *preselect)
 {
 	output_headers(1, 0, 0, 0, 1, 0);
-	do_template("edituser_select", NULL);
+	do_template("aide_edituser_select", NULL);
         end_burst();
 }
 
@@ -391,11 +391,16 @@ int ConditionalFlagINetEmail(StrBuf *Target, WCTemplputParams *TP)
 int ConditionalUserAccess(StrBuf *Target, WCTemplputParams *TP)
 {
 	UserListEntry *ul = (UserListEntry*) CTX;
-
-	if (TP->Tokens->Params[3]->Type == TYPE_LONG)
-		return (TP->Tokens->Params[3]->lvalue == ul->AccessLevel);
-	else
+	
+	if (ul == NULL)
 		return 0;
+
+	return GetTemplateTokenNumber(Target, 
+				      TP, 
+				      3, 
+				      AxNewU)
+		==
+		ul->AccessLevel;
 }
 
 /*
@@ -429,7 +434,7 @@ TRYAGAIN:
 	Stat.highest_found = (-1);
 	Done = 0;
 	/* Search for the user's vCard */
-	if (load_msg_ptrs("MSGS ALL||||1", &Stat) > 0) {
+	if (load_msg_ptrs("MSGS ALL||||1", &Stat, NULL) > 0) {
 		at = GetNewHashPos(WCC->summ, 0);
 		while (GetNextHashPos(WCC->summ, at, &HKLen, &HashKey, &vMsg)) {
 			Msg = (message_summary*) vMsg;		
@@ -466,16 +471,19 @@ TRYAGAIN:
 
 	/* If there's no vcard, create one */
 	if ((*VCMsg == NULL) && (already_tried_creating_one == 0)) {
+		FlushStrBuf(Buf);
 		already_tried_creating_one = 1;
 		serv_puts("ENT0 1|||4");
 		StrBuf_ServGetln(Buf);
-		if (GetServerStatus(Buf, NULL) != 4) {
+		if (GetServerStatus(Buf, NULL) == 4) {
 			serv_puts("Content-type: text/x-vcard");
 			serv_puts("");
 			serv_puts("begin:vcard");
 			serv_puts("end:vcard");
 			serv_puts("000");
 		}
+		else 
+			lprintf(1, "Error while creating user vcard: %s\n", ChrPtr(Buf));
 		goto TRYAGAIN;
 	}
 	FreeStrBuf(&Buf);
@@ -580,7 +588,7 @@ void display_edituser(const char *supplied_username, int is_new) {
 			SubTP.Filter.ContextType = CTX_USERLIST;
 			SubTP.Context = UL;
 			output_headers(1, 0, 0, 0, 1, 0);
-			DoTemplate(HKEY("userlist_detailview"), NULL, &SubTP);
+			DoTemplate(HKEY("aide_edituser_detailview"), NULL, &SubTP);
 			end_burst();
 		}
 		DeleteUserListEntry(UL);
@@ -792,4 +800,12 @@ InitModule_USEREDIT
 			 GroupchangenPosts,
 			 CTX_USERLIST);
 
+	REGISTERTokenParamDefine(AxDeleted);
+	REGISTERTokenParamDefine(AxNewU);
+	REGISTERTokenParamDefine(AxProbU);
+	REGISTERTokenParamDefine(AxLocU);
+	REGISTERTokenParamDefine(AxNetU);
+	REGISTERTokenParamDefine(AxPrefU);
+	REGISTERTokenParamDefine(AxAideU);
 }
+
