@@ -1,5 +1,5 @@
 /*
- * $Id: webcit.c 8082 2009-11-24 22:45:46Z dothebart $
+ * $Id: webcit.c 8507 2010-04-10 10:07:57Z dothebart $
  *
  * This is the main transaction loop of the web service.  It maintains a
  * persistent session to the Citadel server, handling HTTP WebCit requests as
@@ -13,12 +13,6 @@
 #include "webserver.h"
 
 
-/*
- * String to unset the cookie.
- * Any date "in the past" will work, so I chose my birthday, right down to
- * the exact minute.  :)
- */
-static char *unset = "; expires=28-May-1971 18:10:00 GMT";
 StrBuf *csslocal = NULL;
 HashList *HandlerHash = NULL;
 
@@ -47,21 +41,36 @@ void WebcitAddUrlHandler(const char * UrlString, long UrlSLen,
 	Put(HandlerHash, UrlString, UrlSLen, NewHandler, DeleteWebcitHandler);
 }
 
-
+void tmplput_HANDLER_DISPLAYNAME(StrBuf *Target, WCTemplputParams *TP) 
+{
+	wcsession *WCC = WC;
+	if (WCC->Hdr->HR.Handler != NULL)
+		StrBufAppendTemplate(Target, TP, WCC->Hdr->HR.Handler->DisplayName, 0);
+}
 /*
  * web-printing funcion. uses our vsnprintf wrapper
  */
+#ifdef UBER_VERBOSE_DEBUGGING
+void wcc_printf(const char *FILE, const char *FUNCTION, long LINE, const char *format,...)
+#else
 void wc_printf(const char *format,...)
+#endif
 {
 	wcsession *WCC = WC;
 	va_list arg_ptr;
 
 	if (WCC->WBuf == NULL)
 		WCC->WBuf = NewStrBuf();
+#ifdef UBER_VERBOSE_DEBUGGING
+	StrBufAppendPrintf(WCC->WBuf, "\n%s:%s:%d[", FILE, FUNCTION, LINE);
+#endif
 
 	va_start(arg_ptr, format);
 	StrBufVAppendPrintf(WCC->WBuf, format, arg_ptr);
 	va_end(arg_ptr);
+#ifdef UBER_VERBOSE_DEBUGGING
+	StrBufAppendPrintf(WCC->WBuf, "]\n");
+#endif
 }
 
 /*
@@ -118,7 +127,6 @@ void output_headers(	int do_httpheaders,	/* 1 = output HTTP headers
 			int cache		/* 1 = allow browser to cache this page             */
 ) {
 	wcsession *WCC = WC;
-	char cookie[1024];
 	char httpnow[128];
 
 	hprintf("HTTP/1.1 200 OK\n");
@@ -159,28 +167,7 @@ void output_headers(	int do_httpheaders,	/* 1 = output HTTP headers
 		);
 	}
 
-	if (cache < 2) {
-
-		stuff_to_cookie(cookie, 1024, 
-				WCC->wc_session,
-				WCC->wc_username,
-				WCC->wc_password,
-				WCC->wc_roomname,
-				get_selected_language()
-			);
-		
-		if (unset_cookies) {
-			hprintf("Set-cookie: webcit=%s; path=/\r\n", unset);
-		} else {
-			if (server_cookie != NULL) {
-				hprintf("Set-cookie: webcit=%s; path=/ %s\r\n", 
-					cookie, server_cookie);
-			}
-			else {
-				hprintf("Set-cookie: webcit=%s; path=/\r\n", cookie);
-			}
-		}
-	}
+	if (cache < 2) stuff_to_cookie(unset_cookies);
 
 	if (do_htmlhead) {
 		begin_burst();
@@ -486,8 +473,9 @@ void seconds_since_last_gexp(void)
 
 
 
-void ReadPostData(void)
+int ReadPostData(void)
 {
+	int rc;
 	int body_start = 0;
 	wcsession *WCC = WC;
 	StrBuf *content = NULL;
@@ -507,9 +495,12 @@ void ReadPostData(void)
 	body_start = StrLength(content);
 
 	/** Read the entire input data at once. */
-	client_read_to(WCC->Hdr, content, 
-		       WCC->Hdr->HR.ContentLength,
-		       SLEEPING);
+	rc = client_read_to(WCC->Hdr, content, 
+			    WCC->Hdr->HR.ContentLength,
+			    SLEEPING);
+	if (rc < 0)
+		return rc;
+		
 	
 	if (!strncasecmp(ChrPtr(WCC->Hdr->HR.ContentType), "application/x-www-form-urlencoded", 33)) {
 		StrBufCutLeft(content, body_start);
@@ -530,6 +521,7 @@ void ReadPostData(void)
 		content = NULL;
 	}
 	FreeStrBuf(&content);
+	return 1;
 }
 
 
@@ -543,32 +535,57 @@ void ParseREST_URL(void)
 	HashList *Floors;
 	void *vFloor;
 
+	lprintf(1, "parsing rest URL: %s\n", ChrPtr(WCC->Hdr->HR.ReqLine));
+
 	WCC->Directory = NewHash(1, Flathash);
+	WCC->CurrentFloor = NULL;
 
 	Buf = NewStrBuf();
-	while (StrBufExtract_NextToken(WCC->Hdr->HR.ReqLine, 
-				       Buf, &pCh,  '/') >= 0)
+	while (StrBufExtract_NextToken(Buf, WCC->Hdr->HR.ReqLine, &pCh,  '/') >= 0)
 	{
-		Put(WCC->Directory, IKEY(i), Buf, HFreeStrBuf);
-		if (i==0)
-			pFloor = Buf;
+		if (StrLength(Buf) != 0) {
+			/* ignore empty path segments */
+			StrBufUnescape(Buf, 1);
+			Put(WCC->Directory, IKEY(i), Buf, HFreeStrBuf);
+			if (i==0)
+				pFloor = Buf;
+			Buf = NewStrBuf();
+		}
 		i++;
-		Buf = NewStrBuf();
 	}
-	if (i == 0)
-		FreeStrBuf(&Buf);
-	else if (pFloor != NULL)
+
+	FreeStrBuf(&Buf);
+	if (pFloor != NULL)
 	{
 		Floors = GetFloorListHash(NULL, NULL);
 		
 		if (Floors != NULL)
 		{
-			if (GetHash(Floors, SKEY(pFloor), &vFloor))
-				WCC->CurrentFloor = (floor*) vFloor;
+			if (GetHash(WCC->FloorsByName, SKEY(pFloor), &vFloor))
+				WCC->CurrentFloor = (Floor*) vFloor;
 		}
 	}
 }
 
+int Conditional_REST_DEPTH(StrBuf *Target, WCTemplputParams *TP)
+{
+	long Depth, IsDepth;
+	long offset = 0;
+	wcsession *WCC = WC;
+
+	if (WCC->Hdr->HR.Handler != NULL)
+		offset ++;
+	Depth = GetTemplateTokenNumber(Target, TP, 2, 0);
+	IsDepth = GetCount(WCC->Directory) + offset;
+
+//	LogTemplateError(Target, "bla", 1, TP, "REST_DEPTH: %ld : %ld\n", Depth, IsDepth);
+	if (Depth < 0) {
+		Depth = -Depth;
+		return IsDepth > Depth;
+	}
+	else 
+		return Depth == IsDepth;
+}
 
 
 
@@ -586,24 +603,24 @@ void session_loop(void)
 	 * so we can use them to reconnect a timed out session if we have to.
 	 */
 	wcsession *WCC;
-
-	
-	Buf = NewStrBuf();
-
+      
 	WCC= WC;
-
 	WCC->upload_length = 0;
 	WCC->upload = NULL;
 	WCC->is_mobile = 0;
-	WCC->trailing_javascript = NewStrBuf();
 	WCC->Hdr->nWildfireHeaders = 0;
 	if (WCC->Hdr->HR.Handler != NULL)
 		Flags = WCC->Hdr->HR.Handler->Flags; /* so we can temporarily add our own... */
 
 	if (WCC->Hdr->HR.ContentLength > 0) {
-		ReadPostData();
+		if (ReadPostData() < 0) {
+			return;
+		}
 	}
 
+	Buf = NewStrBuf();
+	WCC->trailing_javascript = NewStrBuf();
+
 	/* If there are variables in the URL, we must grab them now */
 	if (WCC->Hdr->PlainArgs != NULL)
 		ParseURLParams(WCC->Hdr->PlainArgs);
@@ -681,11 +698,11 @@ void session_loop(void)
 	 * If we aren't in any room yet, but we have cookie data telling us where we're
 	 * supposed to be, and 'gotofirst' was not specified, then go there.
 	 */
-	else if ( (StrLength(WCC->wc_roomname) == 0) && ( (StrLength(WCC->Hdr->c_roomname) > 0) )) {
+	else if ( (StrLength(WCC->CurRoom.name) == 0) && ( (StrLength(WCC->Hdr->c_roomname) > 0) )) {
 		int ret;
 
 		lprintf(9, "We are in '%s' but cookie indicates '%s', going there...\n",
-			ChrPtr(WCC->wc_roomname),
+			ChrPtr(WCC->CurRoom.name),
 			ChrPtr(WCC->Hdr->c_roomname)
 		);
 		ret = gotoroom(WCC->Hdr->c_roomname);	/* do quietly to avoid session output! */
@@ -697,7 +714,7 @@ void session_loop(void)
 
 	if (WCC->Hdr->HR.Handler != NULL) {
 		if (!WCC->logged_in && ((WCC->Hdr->HR.Handler->Flags & ANONYMOUS) == 0)) {
-			display_login(NULL);
+			display_login();
 		}
 		else {
 /*
@@ -721,7 +738,7 @@ void session_loop(void)
 			if (xhttp)
 				authorization_required();
 			else 
-				display_login(NULL);
+				display_login();
 		}
 		/*
 		 * Toplevel dav requests? or just a flat browser request? 
@@ -752,6 +769,10 @@ void sleeeeeeeeeep(int seconds)
 	select(0, NULL, NULL, NULL, &tv);
 }
 
+int Conditional_IS_HTTPS(StrBuf *Target, WCTemplputParams *TP)
+{
+	return is_https != 0;
+}
 
 int ConditionalImportantMesage(StrBuf *Target, WCTemplputParams *TP)
 {
@@ -809,9 +830,13 @@ InitModule_WEBCIT
 
 	WebcitAddUrlHandler(HKEY("401"), "", 0, authorization_required, ANONYMOUS|COOKIEUNNEEDED);
 	RegisterConditional(HKEY("COND:IMPMSG"), 0, ConditionalImportantMesage, CTX_NONE);
+	RegisterConditional(HKEY("COND:REST:DEPTH"), 0, Conditional_REST_DEPTH, CTX_NONE);
+	RegisterConditional(HKEY("COND:IS_HTTPS"), 0, Conditional_IS_HTTPS, CTX_NONE);
+
 	RegisterNamespace("CSSLOCAL", 0, 0, tmplput_csslocal, NULL, CTX_NONE);
 	RegisterNamespace("IMPORTANTMESSAGE", 0, 0, tmplput_importantmessage, NULL, CTX_NONE);
 	RegisterNamespace("TRAILING_JAVASCRIPT", 0, 0, tmplput_trailing_javascript, NULL, CTX_NONE);
+	RegisterNamespace("URL:DISPLAYNAME", 0, 1, tmplput_HANDLER_DISPLAYNAME, NULL, CTX_NONE);
 
 	snprintf(dir, SIZ, "%s/webcit.css", static_local_dir);
 	if (!access(dir, R_OK)) {
