@@ -1,11 +1,24 @@
-
-/* $Id: webcit.h 8326 2010-02-17 20:24:53Z ajc $ */
+/* $Id: webcit.h 8725 2010-07-28 21:27:42Z ajc $
+ *
+ * Copyright (c) 1987-2010 by the citadel.org team
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
 
 #include "sysdep.h"
-
-
 #include <sys/select.h>
-
 #include <ctype.h>
 #include <stdlib.h>
 #ifdef HAVE_UNISTD_H
@@ -38,7 +51,6 @@
 #include <pthread.h>
 #include <signal.h>
 #include <sys/utsname.h>
-
 #include <libcitadel.h>
 
 #ifndef INADDR_NONE
@@ -89,8 +101,8 @@
 #include "subst.h"
 #include "messages.h"
 #include "paramhandling.h"
-#include "preferences.h"
 #include "roomops.h"
+#include "preferences.h"
 
 #ifdef HAVE_OPENSSL
 /* Work around RedHat's b0rken OpenSSL includes */
@@ -102,6 +114,17 @@ extern char *ssl_cipher_list;
 #define	DEFAULT_SSL_CIPHER_LIST	"DEFAULT"	/* See http://openssl.org/docs/apps/ciphers.html */
 #endif
 
+#if SIZEOF_SIZE_T == SIZEOF_INT 
+#define SIZE_T_FMT "%d"
+#else
+#define SIZE_T_FMT "%ld"
+#endif
+
+#if SIZEOF_LONG_UNSIGNED_INT == SIZEOF_INT
+#define ULONG_FMT "%d"
+#else
+#define ULONG_FMT "%ld"
+#endif
 
 #define CALENDAR_ROOM_NAME	"Calendar"
 #define PRODID "-//Citadel//NONSGML Citadel Calendar//EN"
@@ -115,14 +138,13 @@ extern char *ssl_cipher_list;
 #define PORT_NUM		2000		/* port number to listen on */
 #define DEVELOPER_ID		0
 #define CLIENT_ID		4
-#define CLIENT_VERSION		772		/* This version of WebCit */
-#define MINIMUM_CIT_VERSION	772		/* min required Citadel ver */
-#define	LIBCITADEL_MIN		772		/* min required libcitadel ver */
+#define CLIENT_VERSION		783		/* This version of WebCit */
+#define MINIMUM_CIT_VERSION	781		/* min required Citadel ver */
+#define	LIBCITADEL_MIN		781		/* min required libcitadel ver */
 #define DEFAULT_HOST		"localhost"	/* Default Citadel server */
 #define DEFAULT_PORT		"504"
 #define TARGET			"webcit01"	/* Target for inline URL's */
 #define HOUSEKEEPING		15		/* Housekeeping frequency */
-#define MIN_WORKER_THREADS	5
 #define MAX_WORKER_THREADS	250
 #define LISTEN_QUEUE_LENGTH	100		/* listen() backlog queue */
 
@@ -178,6 +200,7 @@ extern char *ssl_cipher_list;
 #define UA_POSTALLOWED          32
 #define UA_ADMINALLOWED         64
 #define UA_DELETEALLOWED        128
+#define UA_ISTRASH              256 /* Only available in room view... */
 
 
 /*
@@ -201,6 +224,8 @@ extern char *ssl_cipher_list;
 			US_FLOORS | US_COLOR | US_PROMPTCTL )
 
 
+
+
 #define MAJORCODE(a) (((int)(a / 100) ) * 100)
 
 #define LISTING_FOLLOWS	100
@@ -242,6 +267,20 @@ extern char *ssl_cipher_list;
  */
 #define NLI	"(not logged in)"
 
+/*
+ * Expiry policy for the autopurger
+ */
+#define EXPIRE_NEXTLEVEL        0       /* Inherit expiration policy    */
+#define EXPIRE_MANUAL           1       /* Don't expire messages at all */
+#define EXPIRE_NUMMSGS          2       /* Keep only latest n messages  */
+#define EXPIRE_AGE              3       /* Expire messages after n days */
+typedef struct __ExpirePolicy {
+	int loaded; /* has this been loaded from the server? */
+        int expire_mode;
+        int expire_value;
+}ExpirePolicy;
+void LoadExpirePolicy(GPEXWhichPolicy which);
+void SaveExpirePolicyFromHTTP(GPEXWhichPolicy which);
 
 /*
  * Linked list of session variables encoded in an x-www-urlencoded content type
@@ -316,11 +355,20 @@ typedef struct _addrbookent {
 #define COOKIEUNNEEDED (1<<6)
 #define ISSTATIC (1<<7)
 #define FORCE_SESSIONCLOSE (1<<8)
-#define PARSE_REST_URL (1<<0)
+#define PARSE_REST_URL (1<<9)
+#define PROHIBIT_STARTPAGE (1<<10)
 
+typedef enum _RESTDispatchID {
+	ExistsID,
+	PutID,
+	DeleteID
+} RESTDispatchID;
+
+typedef int (*WebcitRESTDispatchID)(RESTDispatchID WhichAction, int IgnoreFloor);
 typedef void (*WebcitHandlerFunc)(void);
 typedef struct  _WebcitHandler{
 	WebcitHandlerFunc F;
+	WebcitRESTDispatchID RID;
 	long Flags;
 	StrBuf *Name;
 	StrBuf *DisplayName;
@@ -359,7 +407,7 @@ enum {
 	eCOPY,
 	eNONE
 };
-const char *ReqStrs[eNONE];
+extern const char *ReqStrs[eNONE];
 
 #define NO_AUTH 0
 #define AUTH_COOKIE 1
@@ -436,7 +484,7 @@ struct wcsession {
 	StrBuf *ReadBuf;                        /* here we keep our stuff while reading linebuffered from the server. */
 	StrBuf *MigrateReadLineBuf;             /* here we buffer legacy server read stuff */
 	const char *ReadPos;                    /* whats our read position in ReadBuf? */
-	int chat_sock;				/* Client socket to Citadel server - for chat */
+	int last_chat_seq;			/* When in chat - last message seq# we saw */
 	time_t lastreq;				/* Timestamp of most recent HTTP */
 	time_t last_pager_check;		/* last time we polled for instant msgs */
 	ServInfo *serv_info;			/* Information about the citserver we're connected to */
@@ -447,13 +495,14 @@ struct wcsession {
 	ParsedHttpHdrs *Hdr;
 	StrBuf *WBuf;                           /* Our output buffer */
 	StrBuf *HBuf;                           /* Our HeaderBuffer */
+	StrBuf *WFBuf;                          /* Wildfire error logging buffer */
 
 	HashList *vars; 			/* HTTP variable substitutions for this page */
 	StrBuf *trailing_javascript;		/* extra javascript to be appended to page */
 	char ImportantMessage[SIZ];
 	StrBuf *ImportantMsg;
 	HashList *Directory;			/* Parts of the directory URL in snippets */
-	const floor *CurrentFloor;              /**< when Parsing REST, which floor are we on? */
+	const Floor *CurrentFloor;              /**< when Parsing REST, which floor are we on? */
 
 /* accounting */
 	StrBuf *wc_username;			/* login name of current user */
@@ -462,7 +511,6 @@ struct wcsession {
 	StrBuf *httpauth_pass;  		/* only for GroupDAV sessions */
 	int axlevel;				/* this user's access level */
 	int is_aide;				/* nonzero == this user is an Aide */
-	int is_room_aide;			/* nonzero == this user is a Room Aide in this room */
 	int connected;				/* nonzero == we are connected to Citadel */
 	int logged_in;				/* nonzero == we are logged in  */
 	int need_regi;				/* This user needs to register. */
@@ -470,7 +518,6 @@ struct wcsession {
 
 /* Preferences */
 	StrBuf *cs_inet_email;  		/* User's preferred Internet addr. */
-	char reply_to[512];			/* reply-to address */
 	HashList *hash_prefs;			/* WebCit preferences for this user */
 	StrBuf *DefaultCharset;                 /* Charset the user preferes */
 	int downloaded_prefs;                   /* Has the client download its prefs yet? */
@@ -478,16 +525,8 @@ struct wcsession {
 	int selected_language;			/* Language selected by user */
 	int time_format_cache;                  /* which timeformat does our user like? */
 
-/* current room related */
-	StrBuf *wc_roomname;			/* Room we are currently in */
-	unsigned room_flags;			/* flags associated with the current room */
-	unsigned room_flags2;			/* flags associated with the current room */
-	int wc_view;				/* view for the current room */
-	int wc_default_view;			/* default view for the current room */
-	int wc_is_trash;			/* nonzero == current room is a Trash folder */
-	int wc_floor;				/* floor number of current room */
-	int is_mailbox;				/* the current room is a private mailbox */
-
+	folder CurRoom;                         /* information about our current room */
+	const folder *ThisRoom;                 /* if REST found a room, remember it here. */
 /* next/previous room thingabob */
 	struct march *march;			/* march mode room list */
 	char ugname[128];			/* where does 'ungoto' take us */
@@ -497,14 +536,16 @@ struct wcsession {
 	HashList *attachments;             	/* list of attachments for 'enter message' */
 	int upload_length;			/* content length of http-uploaded data */
 	StrBuf *upload;				/* pointer to http-uploaded data */
-	char upload_filename[PATH_MAX];		/* filename of http-uploaded data */
+	StrBuf *upload_filename;		/* filename of http-uploaded data */
 	char upload_content_type[256];		/* content type of http-uploaded data */
 
 	int new_mail;				/* user has new mail waiting */
 	int remember_new_mail;			/* last count of new mail messages */
 
 /* Roomiew control */
-	HashList *Floors;                       /* floors our citserver has... */
+	HashList *Floors;                       /* floors our citserver has hashed numeric for quicker access*/
+	HashList *FloorsByName;                 /* same but hashed by its name */
+	HashList *Rooms;                        /* our directory structure as loaded by LKRA */
 	HashList *summ;                         /* list of messages for mailbox summary view */
   /** Perhaps these should be within a struct instead */
 	long startmsg;                          /* message number to start at */
@@ -528,7 +569,7 @@ struct wcsession {
 /* cache stuff for templates. TODO: find a smartrer way */
 	HashList *ServCfg;                      /* cache our server config for editing */
 	HashList *InetCfg;                      /* Our inet server config for editing */
-
+	ExpirePolicy Policy[maxpolicy];
 };
 
 
@@ -544,8 +585,8 @@ void RegisterHeaderHandler(const char *Name, long Len, Header_Evaluator F);
 
 
 enum {
-	S_SELECT,
 	S_SHUTDOWN,
+	S_SPAWNER,
 	MAX_SEMAPHORES
 };
 
@@ -577,19 +618,15 @@ extern int is_https;
 extern int setup_wizard;
 extern char wizard_filename[];
 extern int follow_xff;
-extern int num_threads;
+extern int num_threads_existing;
+extern int num_threads_executing;
 
 void InitialiseSemaphores(void);
 void begin_critical_section(int which_one);
 void end_critical_section(int which_one);
 
-void stuff_to_cookie(char *cookie, size_t clen,
-		int session,
-		StrBuf *user,
-		StrBuf *pass,
-		StrBuf *room,
-		const char *language
-);
+void stuff_to_cookie(int unset_cookie);
+
 void cookie_to_stuff(StrBuf *cookie,
 		int *session,
 		StrBuf *user,
@@ -600,7 +637,7 @@ void cookie_to_stuff(StrBuf *cookie,
 void locate_host(StrBuf *TBuf, int);
 void become_logged_in(const StrBuf *user, const StrBuf *pass, StrBuf *serv_response);
 void openid_manual_create(void);
-void display_login();
+void display_login(void);
 void display_openids(void);
 void do_welcome(void);
 void do_logout(void);
@@ -614,7 +651,7 @@ void CreateMimeStr(void);
 int GetConnected(void);
 void DeleteServInfo(ServInfo **FreeMe);
 int uds_connectsock(char *);
-int tcp_connectsock(char *, char *);
+int tcp_connectsock(char *, int);
 int serv_getln(char *strbuf, int bufsize);
 int StrBuf_ServGetln(StrBuf *buf);
 int GetServerStatus(StrBuf *Line, long* FullState);
@@ -645,7 +682,14 @@ void output_headers(    int do_httpheaders,
 			int suppress_check,
 			int cache);
 void output_custom_content_header(const char *ctype);
+
+#ifdef UBER_VERBOSE_DEBUGGING
+#define wc_printf(...) wcc_printf(__FILE__, __FUNCTION__, __LINE__, __VA_ARGS__)
+void wcc_printf(const char *FILE, const char *FUNCTION, long LINE, const char *format, ...);
+#else 
 void wc_printf(const char *format,...)__attribute__((__format__(__printf__,1,2)));
+#endif
+
 void hprintf(const char *format,...)__attribute__((__format__(__printf__,1,2)));
 void output_static(const char* What);
 
@@ -662,7 +706,7 @@ void embed_main_menu(void);
 
 void do_addrbook_view(addrbookent *addrbook, int num_ab);
 void fetch_ab_name(message_summary *Msg, char **namebuf);
-void display_vcard(StrBuf *Target, StrBuf *vcard_source, char alpha, int full, char **storename, long msgnum);
+void display_vcard(StrBuf *Target, wc_mime_attachment *Mime, char alpha, int full, char **storename, long msgnum);
 void jsonMessageList(void);
 void new_summary_view(void);
 void getseen(void);
@@ -706,8 +750,6 @@ void clear_local_substs(void);
 
 
 int lingering_close(int fd);
-char *memreadline(char *start, char *buf, int maxlen);
-char *memreadlinelen(char *start, char *buf, int maxlen, int *retlen);
 long extract_token(char *dest, const char *source, int parmnum, char separator, int maxlen);
 void remove_token(char *source, int parmnum, char separator);
 StrBuf *load_mimepart(long msgnum, char *partnum);
@@ -726,7 +768,6 @@ void folders(void);
 
 
 
-void display_addressbook(long msgnum, char alpha);
 void offer_start_page(StrBuf *Target, WCTemplputParams *TP);
 void convenience_page(const char *titlebarcolor, const char *titlebarmsg, const char *messagetext);
 void output_html(const char *, int, int, StrBuf *, StrBuf *);
@@ -770,17 +811,17 @@ int ical_ctdl_is_overlap(
 
 extern char *months[];
 extern char *days[];
-int read_server_binary(StrBuf *Ret, size_t total_len, StrBuf *Buf);
+int serv_read_binary(StrBuf *Ret, size_t total_len, StrBuf *Buf);
 int StrBuf_ServGetBLOB(StrBuf *buf, long BlobSize);
 int StrBuf_ServGetBLOBBuffered(StrBuf *buf, long BlobSize);
 int read_server_text(StrBuf *Buf, long *nLines);
-int goto_config_room(StrBuf *Buf);
 long locate_user_vcard_in_this_room(message_summary **VCMsg,
 				    wc_mime_attachment **VCAtt);
 void sleeeeeeeeeep(int);
 void http_transmit_thing(const char *content_type, int is_static);
 long unescape_input(char *buf);
 void do_selected_iconbar(void);
+void check_thread_pool_size(void);
 void spawn_another_worker_thread(void);
 void StrEndTab(StrBuf *Target, int tabnum, int num_tabs);
 void StrBeginTab(StrBuf *Target, int tabnum, int num_tabs);
@@ -791,7 +832,6 @@ void end_tab(int tabnum, int num_tabs);
 void str_wiki_index(char *s);
 long guess_calhourformat(void);
 int get_time_format_cached (void);
-int xtoi(const char *in, size_t len);
 const char *get_selected_language(void);
 void display_wiki_pagelist(void);
 
@@ -859,3 +899,4 @@ void http_datestring(char *buf, size_t n, time_t xtime);
 #define WC_TIMEFORMAT_AMPM 1
 #define WC_TIMEFORMAT_24 2
 
+extern int time_to_die;			/* Nonzero if server is shutting down */
